@@ -111,7 +111,8 @@ typedef struct
 {
 	SV *callback;
 	SV *userdata;
-	xchat_hook *hook;				  /* required for timers */
+	xchat_hook *hook;   /* required for timers */
+	xchat_context *ctx; /* allow timers to remember their context */
 	unsigned int depth;
 } HookData;
 
@@ -211,6 +212,8 @@ static int
 fd_cb (int fd, int flags, void *userdata)
 {
 	HookData *data = (HookData *) userdata;
+	int retVal = 0;
+	int count = 0;
 
 	dSP;
 	ENTER;
@@ -220,18 +223,44 @@ fd_cb (int fd, int flags, void *userdata)
 	XPUSHs (data->userdata);
 	PUTBACK;
 
-	call_sv (data->callback, G_EVAL);
+	count = call_sv (data->callback, G_EVAL);
 	SPAGAIN;
+
 	if (SvTRUE (ERRSV)) {
 		xchat_printf (ph, "Error in fd callback %s", SvPV_nolen (ERRSV));
 		POPs;							  /* remove undef from the top of the stack */
+		retVal = XCHAT_EAT_ALL;
+	} else {
+		if (count != 1) {
+			xchat_print (ph, "Fd handler should only return 1 value.");
+			retVal = XCHAT_EAT_NONE;
+		} else {
+			retVal = POPi;
+			if (retVal == 0) {
+				/* if 0 is returned, the fd is going to get unhooked */
+				PUSHMARK (SP);
+				XPUSHs (sv_2mortal (newSViv (PTR2IV (data->hook))));
+				PUTBACK;
+
+				call_pv ("Xchat::unhook", G_EVAL);
+				SPAGAIN;
+
+				SvREFCNT_dec (data->callback);
+
+				if (data->userdata) {
+					SvREFCNT_dec (data->userdata);
+				}
+				free (data);
+			}
+		}
+
 	}
 
 	PUTBACK;
 	FREETMPS;
 	LEAVE;
 
-	return XCHAT_EAT_ALL;
+	return retVal;
 }
 
 static int
@@ -249,6 +278,9 @@ timer_cb (void *userdata)
 	XPUSHs (data->userdata);
 	PUTBACK;
 
+	if (data->ctx) {
+		xchat_set_context (ph, data->ctx);
+	}
 	count = call_sv (data->callback, G_EVAL);
 	SPAGAIN;
 
@@ -588,12 +620,55 @@ XS (XS_Xchat_get_info)
 				PUSHMARK (SP);
 				XPUSHs (sv_2mortal (temp));
 				PUTBACK;
-				return;
 			}
 		}
 	}
 }
 
+static
+XS (XS_Xchat_context_info)
+{
+	HV *hash;
+	const char *const *fields;
+	const char *field;
+	int i = 0;
+	dXSARGS;
+
+	fields = xchat_list_fields (ph, "channels" );
+	hash = newHV ();
+	sv_2mortal ((SV *) hash);
+	while (fields[i] != NULL) {
+		switch (fields[i][0]) {
+			case 's':
+				field = xchat_list_str (ph, NULL, fields[i] + 1);
+				if (field != NULL) {
+					hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
+						newSVpvn (field, strlen (field)), 0);
+				} else {
+					hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
+						&PL_sv_undef, 0);
+				}
+				break;
+			case 'p':
+				hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
+					newSViv (PTR2IV (xchat_list_str (ph, NULL, fields[i] + 1))), 0);
+				break;
+			case 'i':
+				hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
+					newSVuv (xchat_list_int (ph, NULL, fields[i] + 1)), 0);
+				break;
+			case 't':
+				hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
+					newSVnv (xchat_list_time (ph, NULL, fields[i] + 1)), 0);
+				break;
+		}
+			i++;
+	}
+
+	XPUSHs (newRV_noinc ((SV *) hash));
+	XSRETURN (1);
+}
+
 static
 XS (XS_Xchat_get_prefs)
 {
@@ -614,10 +689,9 @@ XS (XS_Xchat_get_prefs)
 			temp = newSVpv (str, 0);
 			SvUTF8_on (temp);
 			SP -= items;
-			PUSHMARK (SP);
+			sp = mark;
 			XPUSHs (sv_2mortal (temp));
 			PUTBACK;
-			return;
 			break;
 		case 2:
 			XSRETURN_IV (integer);
@@ -678,7 +752,7 @@ XS (XS_Xchat_hook_command)
 	char *name;
 	int pri;
 	SV *callback;
-	char *help_text;
+	char *help_text = NULL;
 	SV *userdata;
 	xchat_hook *hook;
 	HookData *data;
@@ -692,7 +766,13 @@ XS (XS_Xchat_hook_command)
 		name = SvPV_nolen (ST (0));
 		pri = (int) SvIV (ST (1));
 		callback = ST (2);
-		help_text = SvPV_nolen (ST (3));
+
+		/* leave the help text has NULL if the help text is undefined to avoid
+		 * overriding the default help message for builtin commands */
+		if (SvOK(ST (3))) {
+			help_text = SvPV_nolen (ST (3));
+		}
+
 		userdata = ST (4);
 		data = NULL;
 
@@ -781,6 +861,7 @@ XS (XS_Xchat_hook_timer)
 		SvREFCNT_inc (data->callback);
 		data->userdata = sv_mortalcopy (userdata);
 		SvREFCNT_inc (data->userdata);
+		data->ctx = xchat_get_context (ph);
 		hook = xchat_hook_timer (ph, timeout, timer_cb, data);
 		data->hook = hook;
 
@@ -834,6 +915,7 @@ XS (XS_Xchat_hook_fd)
 		data->userdata = sv_mortalcopy (userdata);
 		SvREFCNT_inc (data->userdata);
 		hook = xchat_hook_fd (ph, fd, flags, fd_cb, data);
+		data->hook = hook;
 
 		XSRETURN_IV (PTR2IV (hook));
 	}
@@ -862,8 +944,8 @@ XS (XS_Xchat_unhook)
 				SvREFCNT_dec (userdata->userdata);
 				retCount = 1;
 			}
+			free (userdata);
 		}
-		free (userdata);
 		XSRETURN (retCount);
 	}
 	XSRETURN_EMPTY;
@@ -1026,24 +1108,12 @@ XS (XS_Xchat_get_list)
 					if (field != NULL) {
 						hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
 									 newSVpvn (field, strlen (field)), 0);
-						/*                                              xchat_printf (ph, */
-						/*                                                      "string: %s - %d - %s",  */
-						/*                                                      fields[i]+1, */
-						/*                                                      strlen(fields[i]+1), */
-						/*                                                      field, strlen(field) */
-						/*                                                              ); */
 					} else {
 						hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
 									 &PL_sv_undef, 0);
-						/*                                              xchat_printf (ph, */
-						/*                                                      "string: %s - %d - undef",       */
-						/*                                                              fields[i]+1, */
-						/*                                                              strlen(fields[i]+1) */
-						/*                                                              ); */
 					}
 					break;
 				case 'p':
-					/*                                       xchat_printf (ph, "pointer: %s", fields[i]+1); */
 					hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
 								 newSViv (PTR2IV (xchat_list_str (ph, list,
 																			 fields[i] + 1)
@@ -1053,9 +1123,6 @@ XS (XS_Xchat_get_list)
 					hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
 								 newSVuv (xchat_list_int (ph, list, fields[i] + 1)),
 								 0);
-					/*                                       xchat_printf (ph, "int: %s - %d",fields[i]+1, */
-					/*                                                        xchat_list_int (ph, list, fields[i]+1) */
-					/*                                                       ); */
 					break;
 				case 't':
 					hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
@@ -1112,7 +1179,8 @@ xs_init (pTHX)
 	newXS ("Xchat::Internal::command", XS_Xchat_command, __FILE__);
 	newXS ("Xchat::Internal::set_context", XS_Xchat_set_context, __FILE__);
 	newXS ("Xchat::Internal::get_info", XS_Xchat_get_info, __FILE__);
-
+	newXS ("Xchat::Internal::context_info", XS_Xchat_context_info, __FILE__);
+	
 	newXS ("Xchat::find_context", XS_Xchat_find_context, __FILE__);
 	newXS ("Xchat::get_context", XS_Xchat_get_context, __FILE__);
 	newXS ("Xchat::get_prefs", XS_Xchat_get_prefs, __FILE__);
@@ -1199,18 +1267,38 @@ static int
 perl_load_file (char *filename)
 {
 #ifdef WIN32
-	static HINSTANCE lib = NULL;
+	static HMODULE lib = NULL;
 
 	if (!lib) {
-		lib = LoadLibrary (PERL_DLL);
+		lib = LoadLibraryA (PERL_DLL);
 		if (!lib) {
-			thread_mbox ("Cannot open " PERL_DLL "\n\n"
-							 "You must have ActivePerl installed in order to\n"
-							 "run perl scripts.\n\n"
-							 "http://www.activestate.com/ActivePerl/\n\n"
-							 "Make sure perl's bin directory is in your PATH.");
+			if (GetLastError () == ERROR_BAD_EXE_FORMAT)
+				/* http://forum.xchat.org/viewtopic.php?t=3277 */
+				thread_mbox ("Cannot use this " PERL_DLL "\n\n"
+								 "32-bit ActivePerl is required.");
+			else {
+				/* a lot of people install this old version */
+				lib = LoadLibraryA ("perl56.dll");
+				if (lib) {
+					FreeLibrary (lib);
+					lib = NULL;
+					thread_mbox ("Cannot open " PERL_DLL "\n\n"
+									 "You must have ActivePerl 5.8 installed in order to\n"
+									 "run perl scripts.\n\n"
+									 "I have found Perl 5.6, but that is too old.");
+				} else {
+					thread_mbox ("Cannot open " PERL_DLL "\n\n"
+									 "You must have ActivePerl 5.8 installed in order to\n"
+									 "run perl scripts.\n\n"
+									 "http://www.activestate.com/ActivePerl/\n\n"
+									 "Make sure perl's bin directory is in your PATH.");
+				}
+			}
+			/* failure */
 			return FALSE;
 		}
+
+		/* success */
 		FreeLibrary (lib);
 	}
 #endif
@@ -1252,8 +1340,7 @@ static int
 perl_command_reloadall (char *word[], char *word_eol[], void *userdata)
 {
 	if (my_perl != NULL) {
-		execute_perl (sv_2mortal (newSVpv ("Xchat::Embed::unload_all", 0)), "");
-		perl_auto_load ();
+		execute_perl (sv_2mortal (newSVpv ("Xchat::Embed::reload_all", 0)), "");
 
 		return XCHAT_EAT_XCHAT;
 	}
@@ -1339,6 +1426,8 @@ xchat_plugin_init (xchat_plugin * plugin_handle, char **plugin_name,
 							  0);
 	xchat_hook_command (ph, "reload", XCHAT_PRI_NORM, perl_command_reload, 0,
 							  0);
+	xchat_hook_command (ph, "pl_reload", XCHAT_PRI_NORM, perl_command_reload, 0,
+							  0);
 	xchat_hook_command (ph, "unloadall", XCHAT_PRI_NORM,
 							  perl_command_unloadall, 0, 0);
 	xchat_hook_command (ph, "reloadall", XCHAT_PRI_NORM,
