@@ -1,3 +1,21 @@
+/***************************************************************************
+                           printevents.h  -  TCL plugin header file
+                           -------------------------------------------------
+    begin                : Sat Nov  9 17:31:20 MST 2002
+    copyright            : Copyright 2002-2007 Daniel P. Stasinski
+    email                : mooooooo@avenues.org
+    generated            : Mon May 21 06:03:00 PM MST 2007
+ ***************************************************************************/
+
+/***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
+
 typedef struct {
     char *event;
     char *emit;
@@ -12,9 +30,11 @@ enum
 	XC_TABOPEN,
 	XC_TABCLOSE,
 	XC_TABFOCUS,
+	XC_KEYPRESS,
 	XC_ADDNOTIFY,
 	XC_BANLIST,
 	XC_BANNED,
+	XC_BEEP,
 	XC_CHANGENICK,
 	XC_CHANACTION,
 	XC_HCHANACTION,
@@ -48,6 +68,7 @@ enum
 	XC_CTCPGENC,
 	XC_CTCPSEND,
 	XC_CTCPSND,
+	XC_CTCPSNDC,
 	XC_DCCCHATABORT,
 	XC_DCCCONCHAT,
 	XC_DCCCHATF,
@@ -77,6 +98,7 @@ enum
 	XC_DELNOTIFY,
 	XC_DISCON,
 	XC_FOUNDIP,
+	XC_GENMSG,
 	XC_IGNOREADD,
 	XC_IGNORECHANGE,
 	XC_IGNOREFOOTER,
@@ -103,6 +125,7 @@ enum
 	XC_NOTIFYNUMBER,
 	XC_NOTIFYOFFLINE,
 	XC_NOTIFYONLINE,
+	XC_OPENDIALOG,
 	XC_PART,
 	XC_PARTREASON,
 	XC_PINGREP,
@@ -121,22 +144,27 @@ enum
 	XC_SERVTEXT,
 	XC_STOPCONNECT,
 	XC_TOPIC,
-	XC_TOPICDATE,
 	XC_NEWTOPIC,
+	XC_TOPICDATE,
 	XC_UKNHOST,
 	XC_USERLIMIT,
 	XC_USERSONCHAN,
+	XC_WHOIS_AUTH,
 	XC_WHOIS5,
 	XC_WHOIS2,
 	XC_WHOIS6,
+	XC_WHOIS_ID,
 	XC_WHOIS4,
 	XC_WHOIS4T,
 	XC_WHOIS1,
+	XC_WHOIS_REALHOST,
 	XC_WHOIS3,
+	XC_WHOIS_SPECIAL,
 	XC_UJOIN,
+	XC_UKICK,
 	XC_UPART,
 	XC_UPARTREASON,
-	XC_UKICK,
+	XC_UACTION,
 	XC_UINVITE,
 	XC_UCHANMSG,
 	XC_UCHANGENICK,
@@ -149,12 +177,14 @@ static print_event xc[] = {
 	{ "XC_TABOPEN", "Open Context", -2, NULL },
 	{ "XC_TABCLOSE", "Close Context", -2, NULL },
 	{ "XC_TABFOCUS", "Focus Tab", -2, NULL },
+	{ "XC_KEYPRESS", "Key Press", 4, NULL },
 	{ "XC_ADDNOTIFY", "Add Notify", 1, NULL },
 	{ "XC_BANLIST", "Ban List", 4, NULL },
 	{ "XC_BANNED", "Banned", 1, NULL },
+	{ "XC_BEEP", "Beep", 0, NULL },
 	{ "XC_CHANGENICK", "Change Nick", 2, NULL },
-	{ "XC_CHANACTION", "Channel Action", 2, NULL },
-	{ "XC_HCHANACTION", "Channel Action Hilight", 2, NULL },
+	{ "XC_CHANACTION", "Channel Action", 3, NULL },
+	{ "XC_HCHANACTION", "Channel Action Hilight", 3, NULL },
 	{ "XC_CHANBAN", "Channel Ban", 2, NULL },
 	{ "XC_CHANDATE", "Channel Creation", 2, NULL },
 	{ "XC_CHANDEHOP", "Channel DeHalfOp", 2, NULL },
@@ -164,10 +194,10 @@ static print_event xc[] = {
 	{ "XC_CHANHOP", "Channel Half-Operator", 2, NULL },
 	{ "XC_CHANINVITE", "Channel INVITE", 2, NULL },
 	{ "XC_CHANLISTHEAD", "Channel List", 0, NULL },
-	{ "XC_CHANMSG", "Channel Message", 3, NULL },
+	{ "XC_CHANMSG", "Channel Message", 4, NULL },
 	{ "XC_CHANMODEGEN", "Channel Mode Generic", 4, NULL },
 	{ "XC_CHANMODES", "Channel Modes", 2, NULL },
-	{ "XC_HCHANMSG", "Channel Msg Hilight", 3, NULL },
+	{ "XC_HCHANMSG", "Channel Msg Hilight", 4, NULL },
 	{ "XC_CHANNOTICE", "Channel Notice", 3, NULL },
 	{ "XC_CHANOP", "Channel Operator", 2, NULL },
 	{ "XC_CHANRMEXEMPT", "Channel Remove Exempt", 2, NULL },
@@ -185,8 +215,9 @@ static print_event xc[] = {
 	{ "XC_CTCPGENC", "CTCP Generic to Channel", 3, NULL },
 	{ "XC_CTCPSEND", "CTCP Send", 2, NULL },
 	{ "XC_CTCPSND", "CTCP Sound", 2, NULL },
+	{ "XC_CTCPSNDC", "CTCP Sound to Channel", 3, NULL },
 	{ "XC_DCCCHATABORT", "DCC CHAT Abort", 1, NULL },
-	{ "XC_DCCCONCHAT", "DCC CHAT Connect", 1, NULL },
+	{ "XC_DCCCONCHAT", "DCC CHAT Connect", 2, NULL },
 	{ "XC_DCCCHATF", "DCC CHAT Failed", 4, NULL },
 	{ "XC_DCCCHATOFFER", "DCC CHAT Offer", 1, NULL },
 	{ "XC_DCCCHATOFFERING", "DCC CHAT Offering", 1, NULL },
@@ -201,7 +232,7 @@ static print_event xc[] = {
 	{ "XC_DCCRECVCOMP", "DCC RECV Complete", 4, NULL },
 	{ "XC_DCCCONRECV", "DCC RECV Connect", 3, NULL },
 	{ "XC_DCCRECVERR", "DCC RECV Failed", 4, NULL },
-	{ "XC_DCCFILEERR", "DCC RECV File Open Error", 1, NULL },
+	{ "XC_DCCFILEERR", "DCC RECV File Open Error", 2, NULL },
 	{ "XC_DCCRENAME", "DCC Rename", 2, NULL },
 	{ "XC_DCCRESUMEREQUEST", "DCC RESUME Request", 3, NULL },
 	{ "XC_DCCSENDABORT", "DCC SEND Abort", 2, NULL },
@@ -214,6 +245,7 @@ static print_event xc[] = {
 	{ "XC_DELNOTIFY", "Delete Notify", 1, NULL },
 	{ "XC_DISCON", "Disconnected", 1, NULL },
 	{ "XC_FOUNDIP", "Found IP", 1, NULL },
+	{ "XC_GENMSG", "Generic Message", 2, NULL },
 	{ "XC_IGNOREADD", "Ignore Add", 1, NULL },
 	{ "XC_IGNORECHANGE", "Ignore Changed", 1, NULL },
 	{ "XC_IGNOREFOOTER", "Ignore Footer", 0, NULL },
@@ -238,14 +270,15 @@ static print_event xc[] = {
 	{ "XC_NOTIFYEMPTY", "Notify Empty", 0, NULL },
 	{ "XC_NOTIFYHEAD", "Notify Header", 0, NULL },
 	{ "XC_NOTIFYNUMBER", "Notify Number", 1, NULL },
-	{ "XC_NOTIFYOFFLINE", "Notify Offline", 2, NULL },
-	{ "XC_NOTIFYONLINE", "Notify Online", 2, NULL },
+	{ "XC_NOTIFYOFFLINE", "Notify Offline", 3, NULL },
+	{ "XC_NOTIFYONLINE", "Notify Online", 3, NULL },
+	{ "XC_OPENDIALOG", "Open Dialog", 0, NULL },
 	{ "XC_PART", "Part", 3, NULL },
 	{ "XC_PARTREASON", "Part with Reason", 4, NULL },
 	{ "XC_PINGREP", "Ping Reply", 2, NULL },
 	{ "XC_PINGTIMEOUT", "Ping Timeout", 1, NULL },
-	{ "XC_PRIVMSG", "Private Message", 2, NULL },
-	{ "XC_DPRIVMSG", "Private Message to Dialog", 2, NULL },
+	{ "XC_PRIVMSG", "Private Message", 3, NULL },
+	{ "XC_DPRIVMSG", "Private Message to Dialog", 3, NULL },
 	{ "XC_ALREADYPROCESS", "Process Already Running", 0, NULL },
 	{ "XC_QUIT", "Quit", 3, NULL },
 	{ "XC_RAWMODES", "Raw Modes", 2, NULL },
@@ -258,23 +291,29 @@ static print_event xc[] = {
 	{ "XC_SERVTEXT", "Server Text", 2, NULL },
 	{ "XC_STOPCONNECT", "Stop Connection", 1, NULL },
 	{ "XC_TOPIC", "Topic", 2, NULL },
-	{ "XC_TOPICDATE", "Topic Creation", 3, NULL },
 	{ "XC_NEWTOPIC", "Topic Change", 3, NULL },
+	{ "XC_TOPICDATE", "Topic Creation", 3, NULL },
 	{ "XC_UKNHOST", "Unknown Host", 0, NULL },
 	{ "XC_USERLIMIT", "User Limit", 1, NULL },
 	{ "XC_USERSONCHAN", "Users On Channel", 2, NULL },
+	{ "XC_WHOIS_AUTH", "WhoIs Authenticated", 3, NULL },
 	{ "XC_WHOIS5", "WhoIs Away Line", 2, NULL },
 	{ "XC_WHOIS2", "WhoIs Channel/Oper Line", 2, NULL },
 	{ "XC_WHOIS6", "WhoIs End", 1, NULL },
+	{ "XC_WHOIS_ID", "WhoIs Identified", 2, NULL },
 	{ "XC_WHOIS4", "WhoIs Idle Line", 2, NULL },
 	{ "XC_WHOIS4T", "WhoIs Idle Line with Signon", 3, NULL },
 	{ "XC_WHOIS1", "WhoIs Name Line", 4, NULL },
+	{ "XC_WHOIS_REALHOST", "WhoIs Real Host", 4, NULL },
 	{ "XC_WHOIS3", "WhoIs Server Line", 2, NULL },
+	{ "XC_WHOIS_SPECIAL", "WhoIs Special", 3, NULL },
 	{ "XC_UJOIN", "You Join", 3, NULL },
+	{ "XC_UKICK", "You Kicked", 4, NULL },
 	{ "XC_UPART", "You Part", 3, NULL },
 	{ "XC_UPARTREASON", "You Part with Reason", 4, NULL },
-	{ "XC_UKICK", "You Kicked", 4, NULL },
+	{ "XC_UACTION", "Your Action", 3, NULL },
 	{ "XC_UINVITE", "Your Invitation", 3, NULL },
-	{ "XC_UCHANMSG", "Your Message", 3, NULL },
+	{ "XC_UCHANMSG", "Your Message", 4, NULL },
 	{ "XC_UCHANGENICK", "Your Nick Changing", 2, NULL }
 };
+
