@@ -209,7 +209,20 @@ dcc_remove_from_sum (struct DCC *dcc)
 		dcc_getcpssum -= dcc->cps;
 }
 
-/* this is called from xchat.c:xchat_misc_checks() every 2 seconds. */
+gboolean
+is_dcc (struct DCC *dcc)
+{
+	GSList *list = dcc_list;
+	while (list)
+	{
+		if (list->data == dcc)
+			return TRUE;
+		list = list->next;
+	}
+	return FALSE;
+}
+
+/* this is called from xchat.c:xchat_misc_checks() every 1 second. */
 
 void
 dcc_check_timeouts (void)
@@ -286,6 +299,7 @@ dcc_lookup_proxy (char *host, struct sockaddr_in *addr)
 			return TRUE;
 		}
 		free (cache_host);
+		cache_host = NULL;
 	}
 
 	h = gethostbyname (host);
@@ -459,40 +473,19 @@ struct DCC *
 dcc_write_chat (char *nick, char *text)
 {
 	struct DCC *dcc;
+	int len;
 
 	dcc = find_dcc (nick, "", TYPE_CHATRECV);
 	if (!dcc)
 		dcc = find_dcc (nick, "", TYPE_CHATSEND);
 	if (dcc && dcc->dccstat == STAT_ACTIVE)
 	{
-		char *locale;
-		gsize loc_len;
-		int len;
-
 		len = strlen (text);
-
-		if (dcc->serv->encoding == NULL)	/* system */
-		{
-			locale = NULL;
-			if (!prefs.utf8_locale)
-				locale = g_locale_from_utf8 (text, len, NULL, &loc_len, NULL);
-		} else
-		{
-			locale = g_convert (text, len, dcc->serv->encoding, "UTF-8", 0, &loc_len, 0);
-		}
-
-		if (locale)
-		{
-			text = locale;
-			len = loc_len;
-		}
-
-		dcc->size += len;
-		send (dcc->sok, text, len, 0);
+		tcp_send_real (NULL, dcc->sok, dcc->serv->encoding, dcc->serv->using_irc,
+							text, len);
 		send (dcc->sok, "\n", 1, 0);
+		dcc->size += len;
 		fe_dcc_update (dcc);
-		if (locale)
-			g_free (locale);
 		return dcc;
 	}
 	return 0;
@@ -518,7 +511,9 @@ dcc_chat_line (struct DCC *dcc, char *line)
 	if (dcc->serv->using_cp1255)
 		len++;	/* include the NUL terminator */
 
-	if (dcc->serv->encoding == NULL)     /* system */
+	if (dcc->serv->using_irc) /* using "IRC" encoding (CP1252/UTF-8 hybrid) */
+		utf = NULL;
+	else if (dcc->serv->encoding == NULL)     /* system */
 		utf = g_locale_to_utf8 (line, len, NULL, &utf_len, NULL);
 	else
 		utf = g_convert (line, len, "UTF-8", dcc->serv->encoding, 0, &utf_len, 0);
@@ -710,7 +705,7 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				dcc->destfile_fs = g_strdup (buf);
 
 				old = dcc->destfile;
-				dcc->destfile = g_filename_to_utf8 (buf, -1, 0, 0, 0);
+				dcc->destfile = xchat_filename_to_utf8 (buf, -1, 0, 0, 0);
 
 				EMIT_SIGNAL (XP_TE_DCCRENAME, dcc->serv->front_session,
 								 old, dcc->destfile, NULL, NULL, 0);
@@ -797,7 +792,7 @@ static void
 dcc_open_query (server *serv, char *nick)
 {
 	if (prefs.autodialog)
-		open_query (serv, nick);
+		open_query (serv, nick, FALSE);
 }
 
 static gboolean
@@ -1504,30 +1499,13 @@ abortit:
 }
 
 static gboolean
-dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
+dcc_handle_new_ack (struct DCC *dcc)
 {
-	int len;
 	guint32 ack;
 	char buf[16];
-	int sok = dcc->sok;
+	gboolean done = FALSE;
 
-	len = recv (sok, (char *) &ack, 4, MSG_PEEK);
-	if (len < 1)
-	{
-		if (len < 0)
-		{
-			if (would_block ())
-				return TRUE;
-		}
-		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
-						 file_part (dcc->file), dcc->nick,
-						 errorstring ((len < 0) ? sock_error () : 0), NULL, 0);
-		dcc_close (dcc, STAT_FAILED, FALSE);
-		return TRUE;
-	}
-	if (len < 4)
-		return TRUE;
-	recv (sok, (char *) &ack, 4, 0);
+	memcpy (&ack, dcc->ack_buf, 4);
 	dcc->ack = ntohl (ack);
 
 	/* this could mess up when xfering >32bit files */
@@ -1549,6 +1527,7 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		sprintf (buf, "%d", dcc->cps);
 		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
+		done = TRUE;
 	}
 	else if ((!dcc->fastsend) && (dcc->ack >= (dcc->pos & 0xffffffff)))
 	{
@@ -1562,7 +1541,41 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	/* dcc->ack is only used for CPS and PERCENTAGE calcs from now on... */
 #endif
 
-	return TRUE;
+	return done;
+}
+
+static gboolean
+dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
+{
+	int len;
+
+	while (1)
+	{
+		/* try to fill up 4 bytes */
+		len = recv (dcc->sok, dcc->ack_buf, 4 - dcc->ack_pos, 0);
+		if (len < 1)
+		{
+			if (len < 0)
+			{
+				if (would_block ())	/* ok - keep waiting */
+					return TRUE;
+			}
+			EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
+							 file_part (dcc->file), dcc->nick,
+							 errorstring ((len < 0) ? sock_error () : 0), NULL, 0);
+			dcc_close (dcc, STAT_FAILED, FALSE);
+			return TRUE;
+		}
+
+		dcc->ack_pos += len;
+		if (dcc->ack_pos >= 4)
+		{
+			dcc->ack_pos = 0;
+			if (dcc_handle_new_ack (dcc))
+				return TRUE;
+		}
+		/* loop again until would_block() returns true */
+	}
 }
 
 static gboolean
@@ -1773,7 +1786,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 		free (file);
 
 		/* for_files() will use opendir, so we need local FS encoding */
-		path_fs = g_filename_from_utf8 (path, -1, 0, 0, 0);
+		path_fs = xchat_filename_from_utf8 (path, -1, 0, 0, 0);
 		if (path_fs)
 		{
 			recursive = TRUE;
@@ -1792,7 +1805,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 	dcc->maxcps = maxcps;
 
 	/* get the local filesystem encoding */
-	file_fs = g_filename_from_utf8 (file, -1, 0, 0, 0);
+	file_fs = xchat_filename_from_utf8 (file, -1, 0, 0, 0);
 
 	if (stat (file_fs, &st) != -1)
 	{
@@ -1801,78 +1814,78 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 		if (sizeof (st.st_size) > 4 && st.st_size > 4294967295U)
 		{
 			PrintText (sess, "Cannot send files larger than 4 GB.\n");
-			g_free (file_fs);
-			dcc_close (dcc, 0, TRUE);
-			return;
+			goto xit;
 		}
 #endif
 
-		if (*file_part (file_fs) && !S_ISDIR (st.st_mode))
+		if (!(*file_part (file_fs)) || S_ISDIR (st.st_mode) || st.st_size < 1)
 		{
-			if (st.st_size > 0)
+			PrintText (sess, "Cannot send directories or empty files.\n");
+			goto xit;
+		}
+
+		dcc->starttime = dcc->offertime = time (0);
+		dcc->serv = sess->server;
+		dcc->dccstat = STAT_QUEUED;
+		dcc->size = st.st_size;
+		dcc->type = TYPE_SEND;
+		dcc->fp = open (file_fs, OFLAGS | O_RDONLY);
+		if (dcc->fp != -1)
+		{
+			g_free (file_fs);
+			if (passive || dcc_listen_init (dcc, sess))
 			{
-				dcc->starttime = dcc->offertime = time (0);
-				dcc->serv = sess->server;
-				dcc->dccstat = STAT_QUEUED;
-				dcc->size = st.st_size;
-				dcc->type = TYPE_SEND;
-				dcc->fp = open (file_fs, OFLAGS | O_RDONLY);
-				if (dcc->fp != -1)
+				char havespaces = 0;
+				file = dcc->file;
+				while (*file)
 				{
-					g_free (file_fs);
-					if (passive || dcc_listen_init (dcc, sess))
+					if (*file == ' ')
 					{
-						char havespaces = 0;
-						file = dcc->file;
-						while (*file)
-						{
-							if (*file == ' ')
-							{
-								if (prefs.dcc_send_fillspaces)
-						    		*file = '_';
-							  	else
-							   	havespaces = 1;
-							}
-							file++;
-						}
-						dcc->nick = strdup (to);
-						if (prefs.autoopendccsendwindow)
-						{
-							if (fe_dcc_open_send_win (TRUE))	/* already open? add */
-								fe_dcc_add (dcc);
-						} else
-							fe_dcc_add (dcc);
-
-						if (passive)
-						{
-							dcc->pasvid = new_id();
-							snprintf (outbuf, sizeof (outbuf), (havespaces) ?
-									"DCC SEND \"%s\" 199 %d %"DCC_SFMT" %d" :
-									"DCC SEND %s 199 %d %"DCC_SFMT" %d",
-									file_part (dcc->file),
-									0, dcc->size, dcc->pasvid);
-						} else
-						{
-							snprintf (outbuf, sizeof (outbuf), (havespaces) ?
-									"DCC SEND \"%s\" %u %d %"DCC_SFMT :
-									"DCC SEND %s %u %d %"DCC_SFMT,
-									file_part (dcc->file), dcc->addr,
-									dcc->port, dcc->size);
-						}
-						sess->server->p_ctcp (sess->server, to, outbuf);
-
-						EMIT_SIGNAL (XP_TE_DCCOFFER, sess, file_part (dcc->file),
-										 to, dcc->file, NULL, 0);
-					} else
-					{
-						dcc_close (dcc, 0, TRUE);
+						if (prefs.dcc_send_fillspaces)
+				    		*file = '_';
+					  	else
+					   	havespaces = 1;
 					}
-					return;
+					file++;
 				}
+				dcc->nick = strdup (to);
+				if (prefs.autoopendccsendwindow)
+				{
+					if (fe_dcc_open_send_win (TRUE))	/* already open? add */
+						fe_dcc_add (dcc);
+				} else
+					fe_dcc_add (dcc);
+
+				if (passive)
+				{
+					dcc->pasvid = new_id();
+					snprintf (outbuf, sizeof (outbuf), (havespaces) ?
+							"DCC SEND \"%s\" 199 0 %"DCC_SFMT" %d" :
+							"DCC SEND %s 199 0 %"DCC_SFMT" %d",
+							file_part (dcc->file),
+							dcc->size, dcc->pasvid);
+				} else
+				{
+					snprintf (outbuf, sizeof (outbuf), (havespaces) ?
+							"DCC SEND \"%s\" %u %d %"DCC_SFMT :
+							"DCC SEND %s %u %d %"DCC_SFMT,
+							file_part (dcc->file), dcc->addr,
+							dcc->port, dcc->size);
+				}
+				sess->server->p_ctcp (sess->server, to, outbuf);
+
+				EMIT_SIGNAL (XP_TE_DCCOFFER, sess, file_part (dcc->file),
+								 to, dcc->file, NULL, 0);
+			} else
+			{
+				dcc_close (dcc, 0, TRUE);
 			}
+			return;
 		}
 	}
 	PrintTextf (sess, _("Cannot access %s\n"), dcc->file);
+	PrintTextf (sess, "%s %d: %s\n", _("Error"), errno, errorstring (errno));
+xit:
 	g_free (file_fs);
 	dcc_close (dcc, 0, TRUE);
 }
@@ -1971,7 +1984,7 @@ is_same_file (struct DCC *dcc, struct DCC *new_dcc)
 
 	/* now handle case-insensitive Filesystems: HFS+, FAT */
 #ifdef WIN32
-#error implement me!
+#warning no win32 implementation - behaviour may be unreliable
 #else
 	/* this fstat() shouldn't really fail */
 	if ((dcc->fp == -1 ? stat (dcc->destfile_fs, &st_a) : fstat (dcc->fp, &st_a)) == -1)
@@ -2083,7 +2096,7 @@ dcc_get_with_destfile (struct DCC *dcc, char *file)
 
 	/* get the local filesystem encoding */
 	g_free (dcc->destfile_fs);
-	dcc->destfile_fs = g_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
+	dcc->destfile_fs = xchat_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
 
 	/* since destfile changed, must check resumability again */
 	is_resumable (dcc);
@@ -2335,7 +2348,7 @@ dcc_add_file (session *sess, char *file, DCC_SIZE size, int port, char *nick, gu
 		strcat (dcc->destfile, file);
 
 		/* get the local filesystem encoding */
-		dcc->destfile_fs = g_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
+		dcc->destfile_fs = xchat_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
 
 		dcc->resumable = 0;
 		dcc->pos = 0;
