@@ -271,11 +271,9 @@ ignore_load ()
 	struct ignore *ignore;
 	struct stat st;
 	char *cfg, *my_cfg;
-	char file[256];
 	int fh, i;
 
-	snprintf (file, sizeof (file), "%s/ignore.conf", get_xdir_fs ());
-	fh = open (file, O_RDONLY | OFLAGS);
+	fh = xchat_open_file ("ignore.conf", O_RDONLY, 0, 0);
 	if (fh != -1)
 	{
 		fstat (fh, &st);
@@ -310,8 +308,7 @@ ignore_save ()
 	GSList *temp = ignore_list;
 	struct ignore *ig;
 
-	snprintf (buf, sizeof (buf), "%s/ignore.conf", get_xdir_fs ());
-	fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
+	fh = xchat_open_file ("ignore.conf", O_TRUNC | O_WRONLY | O_CREAT, 0600, XOF_DOMODE);
 	if (fh != -1)
 	{
 		while (temp)
@@ -330,6 +327,13 @@ ignore_save ()
 
 }
 
+static gboolean
+flood_autodialog_timeout (gpointer data)
+{
+	prefs.autodialog = 1;
+	return FALSE;
+}
+
 int
 flood_check (char *nick, char *ip, server *serv, session *sess, int what)	/*0=ctcp  1=priv */
 {
@@ -395,7 +399,7 @@ flood_check (char *nick, char *ip, server *serv, session *sess, int what)	/*0=ct
 				if (serv->msg_counter == prefs.msg_number_limit)	/*if we reached the maximun numbers of ctcp in the seconds limits */
 				{
 					snprintf (buf, sizeof (buf),
-					 _("You are being MSG flooded from %s, setting autodialog OFF.\n"),
+					 _("You are being MSG flooded from %s, setting gui_auto_open_dialog OFF.\n"),
 								 ip);
 					PrintText (sess, buf);
 					serv->msg_last_time = current_time;	/*we got the flood, restore all the vars for next one */
@@ -403,8 +407,13 @@ flood_check (char *nick, char *ip, server *serv, session *sess, int what)	/*0=ct
 					/*ignore_add (char *mask, int priv, int noti, int chan,
 					   int ctcp, int invi, int unignore, int no_save) */
 
-					/*FIXME: only ignore ctcp or all?, its ignoring ctcps for now */
-					prefs.autodialog = 0;
+					if (prefs.autodialog)
+					{
+						/*FIXME: only ignore ctcp or all?, its ignoring ctcps for now */
+						prefs.autodialog = 0;
+						/* turn it back on in 30 secs */
+						fe_timeout_add (30000, flood_autodialog_timeout, NULL);
+					}
 					return 0;
 				}
 			}
