@@ -22,6 +22,7 @@
 #include <time.h>					  /* asctime() */
 #include <string.h>				  /* strncpy() */
 #include "ssl.h"					  /* struct cert_info */
+#include "inet.h"
 #include "../../config.h"		  /* HAVE_SNPRINTF */
 
 #ifndef HAVE_SNPRINTF
@@ -79,7 +80,6 @@ _SSL_context_init (void (*info_cb_func), int server)
 
 #ifdef WIN32
 	/* under win32, OpenSSL needs to be seeded with some randomness */
-	srand (time (0));
 	for (i = 0; i < 128; i++)
 	{
 		r = rand ();
@@ -101,8 +101,8 @@ ASN1_TIME_snprintf (char *buf, int buf_len, ASN1_TIME * tm)
 	buf[0] = 0;
 	if (expires != NULL)
 	{
-		strncpy (buf, expires, buf_len);
-		buf[buf_len - 1] = 0;
+		memset (buf, 0, buf_len);
+		strncpy (buf, expires, 24);
 	}
 	BIO_free (inMem);
 }
@@ -165,11 +165,11 @@ _SSL_get_cert_info (struct cert_info *cert_info, SSL * ssl)
 	peer_pkey = X509_get_pubkey (peer_cert);
 
 	strncpy (cert_info->algorithm,
-				(alg == NID_undef) ? "UNKNOWN" : OBJ_nid2ln (alg),
+				(alg == NID_undef) ? "Unknown" : OBJ_nid2ln (alg),
 				sizeof (cert_info->algorithm));
 	cert_info->algorithm_bits = EVP_PKEY_bits (peer_pkey);
 	strncpy (cert_info->sign_algorithm,
-				(sign_alg == NID_undef) ? "UNKNOWN" : OBJ_nid2ln (sign_alg),
+				(sign_alg == NID_undef) ? "Unknown" : OBJ_nid2ln (sign_alg),
 				sizeof (cert_info->sign_algorithm));
 	/* EVP_PKEY_bits(ca_pkey)); */
 	cert_info->sign_algorithm_bits = 0;
@@ -188,6 +188,7 @@ _SSL_get_cert_info (struct cert_info *cert_info, SSL * ssl)
 	} else
 		fprintf(stderr, "REMOTE SIDE DOESN'T PROVIDES ->peer_rsa_tmp\n");
 */
+	cert_info->rsa_tmp_bits = 0;
 
 	X509_free (peer_cert);
 
@@ -257,7 +258,8 @@ _SSL_recv (SSL * ssl, char *buf, int len)
 		break;
 	case SSL_ERROR_SYSCALL:
 		/* ??? */
-		perror ("SSL_read/read");
+		if (!would_block ())
+			perror ("SSL_read/read");
 		break;
 	case SSL_ERROR_ZERO_RETURN:
 		/* fprintf(stdeerr, "SSL closed on read\n"); */
