@@ -159,7 +159,7 @@ chanlist_data_free (server *serv)
 			  rows = rows->next)
 		{
 			data = rows->data;
-			free (data->topic);
+			g_free (data->topic);
 			g_free (data->collation_key);
 			free (data);
 		}
@@ -355,6 +355,10 @@ chanlist_build_gui_list (server *serv)
 
 	custom_list_clear ((CustomList *)GET_MODEL (serv));
 
+	/* discard pending rows FIXME: free the structs? */
+	g_slist_free (serv->gui->chanlist_pending_rows);
+	serv->gui->chanlist_pending_rows = NULL;
+
 	/* Reset the counters */
 	chanlist_reset_counters (serv);
 
@@ -602,6 +606,7 @@ chanlist_button_cb (GtkTreeView *tree, GdkEventButton *event, server *serv)
 	GtkWidget *menu;
 	GtkTreeSelection *sel;
 	GtkTreePath *path;
+	char *chan;
 
 	if (event->button != 3)
 		return FALSE;
@@ -629,6 +634,11 @@ chanlist_button_cb (GtkTreeView *tree, GdkEventButton *event, server *serv)
 								chanlist_copychannel, serv);
 	mg_create_icon_item (_("Copy _Topic Text"), GTK_STOCK_COPY, menu,
 								chanlist_copytopic, serv);
+
+	chan = chanlist_get_selected (serv, FALSE);
+	menu_addfavoritemenu (serv, menu, chan);
+	g_free (chan);
+
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
 
 	return TRUE;
