@@ -129,8 +129,9 @@ static const GOptionEntry gopt_entries[] =
  {"plugindir",	'p', 0, G_OPTION_ARG_NONE,	&arg_show_autoload, N_("Show plugin auto-load directory"), NULL},
  {"configdir",	'u', 0, G_OPTION_ARG_NONE,	&arg_show_config, N_("Show user config directory"), NULL},
  {"url",	 0,  0, G_OPTION_ARG_STRING,	&arg_url, N_("Open an irc://server:port/channel URL"), "URL"},
-#ifndef WIN32
- {"existing",	'e', 0, G_OPTION_ARG_NONE,	&arg_existing, N_("Open URL in an existing XChat"), NULL},
+#ifndef WIN32	/* uses DBUS */
+ {"command",	'c', 0, G_OPTION_ARG_STRING,	&arg_command, N_("Execute command:"), "COMMAND"},
+ {"existing",	'e', 0, G_OPTION_ARG_NONE,	&arg_existing, N_("Open URL or execute command in an existing XChat"), NULL},
 #endif
  {"minimize",	 0,  0, G_OPTION_ARG_INT,	&arg_minimize, N_("Begin minimized. Level 0=Normal 1=Iconified 2=Tray"), N_("level")},
  {"version",	'v', 0, G_OPTION_ARG_NONE,	&arg_show_version, N_("Show version information"), NULL},
@@ -357,7 +358,7 @@ fe_idle (gpointer data)
 	if (arg_minimize == 1)
 		gtk_window_iconify (GTK_WINDOW (sess->gui->window));
 	else if (arg_minimize == 2)
-		tray_toggle_visibility ();
+		tray_toggle_visibility (FALSE);
 
 	return 0;
 }
@@ -463,17 +464,17 @@ fe_input_add (int sok, int flags, void *func, void *data)
 }
 
 void
-fe_set_topic (session *sess, char *topic)
+fe_set_topic (session *sess, char *topic, char *stripped_topic)
 {
 	if (!sess->gui->is_tab || sess == current_tab)
 	{
-		gtk_entry_set_text (GTK_ENTRY (sess->gui->topic_entry), topic);
+		gtk_entry_set_text (GTK_ENTRY (sess->gui->topic_entry), stripped_topic);
 		mg_set_topic_tip (sess);
 	} else
 	{
 		if (sess->res->topic_text)
 			free (sess->res->topic_text);
-		sess->res->topic_text = strdup (topic);
+		sess->res->topic_text = strdup (stripped_topic);
 	}
 }
 
@@ -548,9 +549,9 @@ fe_notify_update (char *name)
 }
 
 void
-fe_text_clear (struct session *sess)
+fe_text_clear (struct session *sess, int lines)
 {
-	gtk_xtext_clear (sess->res->buffer);
+	gtk_xtext_clear (sess->res->buffer, lines);
 }
 
 void
@@ -594,12 +595,12 @@ fe_progressbar_end (server *serv)
 }
 
 void
-fe_print_text (struct session *sess, char *text)
+fe_print_text (struct session *sess, char *text, time_t stamp)
 {
-	PrintTextRaw (sess->res->buffer, (unsigned char *)text, prefs.indent_nicks);
+	PrintTextRaw (sess->res->buffer, (unsigned char *)text, prefs.indent_nicks, stamp);
 
 	if (!sess->new_data && sess != current_tab &&
-		 sess->gui->is_tab && !sess->nick_said)
+		 sess->gui->is_tab && !sess->nick_said && stamp == 0)
 	{
 		sess->new_data = TRUE;
 		if (sess->msg_said)
@@ -660,7 +661,7 @@ fe_set_lag (server *serv, int lag)
 	session *sess;
 	gdouble per;
 	char lagtext[64];
-	char lagtip[64];
+	char lagtip[128];
 	unsigned long nowtim;
 
 	if (lag == -1)
@@ -671,7 +672,7 @@ fe_set_lag (server *serv, int lag)
 		lag = (nowtim - serv->lag_sent) / 100000;
 	}
 
-	per = (double)((double)lag / (double)40);
+	per = (double)((double)lag / (double)10);
 	if (per > 1.0)
 		per = 1.0;
 
@@ -716,8 +717,8 @@ fe_set_throttle (server *serv)
 	GSList *list = sess_list;
 	struct session *sess;
 	float per;
-	char tbuf[64];
-	char tip[64];
+	char tbuf[96];
+	char tip[160];
 
 	per = (float) serv->sendq_len / 1024.0;
 	if (per > 1.0)
@@ -757,31 +758,31 @@ fe_set_throttle (server *serv)
 }
 
 void
-fe_ctrl_gui (session *sess, int action, int arg)
+fe_ctrl_gui (session *sess, fe_gui_action action, int arg)
 {
 	switch (action)
 	{
-	case 0:
+	case FE_GUI_HIDE:
 		gtk_widget_hide (sess->gui->window); break;
-	case 1:
+	case FE_GUI_SHOW:
 		gtk_widget_show (sess->gui->window);
 		gtk_window_present (GTK_WINDOW (sess->gui->window));
 		break;
-	case 2:
+	case FE_GUI_FOCUS:
 		mg_bring_tofront_sess (sess); break;
-	case 3:
+	case FE_GUI_FLASH:
 		fe_flash_window (sess); break;
-	case 4:
+	case FE_GUI_COLOR:
 		fe_set_tab_color (sess, arg); break;
-	case 5:
+	case FE_GUI_ICONIFY:
 		gtk_window_iconify (GTK_WINDOW (sess->gui->window)); break;
-	case 6:
+	case FE_GUI_MENU:
 		menu_bar_toggle ();	/* toggle menubar on/off */
 		break;
-	case 7:
+	case FE_GUI_ATTACH:
 		mg_detach (sess, arg);	/* arg: 0=toggle 1=detach 2=attach */
 		break;
-	case 8:
+	case FE_GUI_APPLY:
 		setup_apply_real (TRUE, TRUE);
 	}
 }
@@ -929,7 +930,7 @@ try_browser (const char *browser, const char *arg, const char *url)
 #endif
 
 static void
-fe_open_url_locale (const char *url)
+fe_open_url_inner (const char *url)
 {
 #ifdef WIN32
 	ShellExecute (0, "open", url, NULL, NULL, SW_SHOWNORMAL);
@@ -961,6 +962,21 @@ fe_open_url_locale (const char *url)
 #endif
 }
 
+static void
+fe_open_url_locale (const char *url)
+{
+#ifndef WIN32
+	if (url[0] != '/' && strchr (url, ':') == NULL)
+	{
+		url = g_strdup_printf ("http://%s", url);
+		fe_open_url_inner (url);
+		g_free ((char *)url);
+		return;
+	}
+#endif
+	fe_open_url_inner (url);
+}
+
 void
 fe_open_url (const char *url)
 {
