@@ -1,5 +1,14 @@
 #include "../../config.h"
 
+#ifdef WIN32
+/* If you're compiling this for Windows, your release is un-official
+ * and not condoned. Please don't use the XChat name. Call it YChat
+ * or make up your own name! */
+#define DISPLAY_NAME "YChat"
+#else
+#define DISPLAY_NAME "XChat"
+#endif
+
 #ifndef WIN32
 #include <sys/types.h>
 #include <regex.h>
