@@ -376,8 +376,14 @@ mg_set_access_icon (session_gui *gui, GdkPixbuf *pix, gboolean away)
 {
 	if (gui->op_xpm)
 	{
+		if (pix == gtk_image_get_pixbuf (GTK_IMAGE (gui->op_xpm))) /* no change? */
+		{
+			mg_set_myself_away (gui, away);
+			return;
+		}
+
 		gtk_widget_destroy (gui->op_xpm);
-		gui->op_xpm = 0;
+		gui->op_xpm = NULL;
 	}
 
 	if (pix)
@@ -496,23 +502,23 @@ fe_set_title (session *sess)
 	switch (type)
 	{
 	case SESS_DIALOG:
-		snprintf (tbuf, sizeof (tbuf), "XChat: %s %s @ %s",
+		snprintf (tbuf, sizeof (tbuf), DISPLAY_NAME": %s %s @ %s",
 					 _("Dialog with"), sess->channel, server_get_network (sess->server, TRUE));
 		break;
 	case SESS_SERVER:
-		snprintf (tbuf, sizeof (tbuf), "XChat: %s @ %s",
+		snprintf (tbuf, sizeof (tbuf), DISPLAY_NAME": %s @ %s",
 					 sess->server->nick, server_get_network (sess->server, TRUE));
 		break;
 	case SESS_CHANNEL:
 		/* don't display keys in the titlebar */
 		if ((!(prefs.gui_tweaks & 16)) && has_key (sess->current_modes))
 			snprintf (tbuf, sizeof (tbuf),
-						 "XChat: %s @ %s / %s",
+						 DISPLAY_NAME": %s @ %s / %s",
 						 sess->server->nick, server_get_network (sess->server, TRUE),
 						 sess->channel);
 		else
 			snprintf (tbuf, sizeof (tbuf),
-						 "XChat: %s @ %s / %s (%s)",
+						 DISPLAY_NAME": %s @ %s / %s (%s)",
 						 sess->server->nick, server_get_network (sess->server, TRUE),
 						 sess->channel, sess->current_modes ? sess->current_modes : "");
 		if (prefs.gui_tweaks & 1)
@@ -520,12 +526,12 @@ fe_set_title (session *sess)
 		break;
 	case SESS_NOTICES:
 	case SESS_SNOTICES:
-		snprintf (tbuf, sizeof (tbuf), "XChat: %s @ %s (notices)",
+		snprintf (tbuf, sizeof (tbuf), DISPLAY_NAME": %s @ %s (notices)",
 					 sess->server->nick, server_get_network (sess->server, TRUE));
 		break;
 	default:
 	def:
-		gtk_window_set_title (GTK_WINDOW (sess->gui->window), "XChat");
+		gtk_window_set_title (GTK_WINDOW (sess->gui->window), DISPLAY_NAME);
 		return;
 	}
 
@@ -977,7 +983,6 @@ mg_populate (session *sess)
 		render = FALSE;
 
 	gtk_xtext_buffer_show (GTK_XTEXT (gui->xtext), res->buffer, render);
-	GTK_XTEXT (gui->xtext)->color_paste = sess->color_paste;
 
 	if (gui->is_tab)
 		gtk_widget_set_sensitive (gui->menu, TRUE);
@@ -1183,39 +1188,6 @@ mg_tab_close (session *sess)
 	}
 }
 
-static void
-mg_colorpaste_cb (GtkCheckMenuItem *item, session *sess)
-{
-	sess->color_paste = FALSE;
-	if (item->active)
-		sess->color_paste = TRUE;
-	GTK_XTEXT (sess->gui->xtext)->color_paste = sess->color_paste;
-}
-
-static void
-mg_traymsg_cb (GtkCheckMenuItem *item, session *sess)
-{
-	sess->tray = FALSE;
-	if (item->active)
-		sess->tray = TRUE;
-}
-
-static void
-mg_beepmsg_cb (GtkCheckMenuItem *item, session *sess)
-{
-	sess->beep = FALSE;
-	if (item->active)
-		sess->beep = TRUE;
-}
-
-static void
-mg_hidejp_cb (GtkCheckMenuItem *item, session *sess)
-{
-	sess->hide_join_part = TRUE;
-	if (item->active)
-		sess->hide_join_part = FALSE;
-}
-
 static void
 mg_menu_destroy (GtkWidget *menu, gpointer userdata)
 {
@@ -1380,6 +1352,12 @@ mg_open_quit_dialog (gboolean minimize_button)
 			prefs.gui_tray_flags |= 1;
 			/*prefs.gui_quit_dialog = 0;*/
 		}
+		/* force tray icon ON, if not already */
+		if (!prefs.gui_tray)
+		{
+			prefs.gui_tray = 1;
+			tray_apply_setup ();
+		}
 		tray_toggle_visibility (TRUE);
 		break;
 	}
@@ -1592,63 +1570,97 @@ mg_create_color_menu (GtkWidget *menu, session *sess)
 	}
 }
 
-static gboolean
-mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventButton *event)
+static void
+mg_set_guint8 (GtkCheckMenuItem *item, guint8 *setting)
+{
+	session *sess = current_sess;
+	guint8 logging = sess->text_logging;
+
+	*setting = SET_OFF;
+	if (item->active)
+		*setting = SET_ON;
+
+	/* has the logging setting changed? */
+	if (logging != sess->text_logging)
+		log_open_or_close (sess);
+}
+
+static void
+mg_perchan_menu_item (char *label, GtkWidget *menu, guint8 *setting, guint global)
+{
+	guint8 initial_value = *setting;
+
+	/* if it's using global value, use that as initial state */
+	if (initial_value == SET_DEFAULT)
+		initial_value = global;
+
+	menu_toggle_item (label, menu, mg_set_guint8, setting, initial_value);
+}
+
+static void
+mg_create_perchannelmenu (session *sess, GtkWidget *menu)
+{
+	GtkWidget *submenu;
+
+	submenu = menu_quick_sub (_("_Settings"), menu, NULL, XCMENU_MNEMONIC, -1);
+
+	mg_perchan_menu_item (_("_Log to Disk"), submenu, &sess->text_logging, prefs.logging);
+	mg_perchan_menu_item (_("_Reload Scrollback"), submenu, &sess->text_scrollback, prefs.text_replay);
+	if (sess->type == SESS_CHANNEL)
+		mg_perchan_menu_item (_("_Hide Join/Part Messages"), submenu, &sess->text_hidejoinpart, prefs.confmode);
+}
+
+static void
+mg_create_alertmenu (session *sess, GtkWidget *menu)
+{
+	GtkWidget *submenu;
+
+	submenu = menu_quick_sub (_("_Extra Alerts"), menu, NULL, XCMENU_MNEMONIC, -1);
+
+	mg_perchan_menu_item (_("Beep on _Message"), submenu, &sess->alert_beep, prefs.input_beep_chans);
+	mg_perchan_menu_item (_("Blink Tray _Icon"), submenu, &sess->alert_tray, prefs.input_tray_chans);
+	mg_perchan_menu_item (_("Blink Task _Bar"), submenu, &sess->alert_taskbar, prefs.input_flash_chans);
+}
+
+static void
+mg_create_tabmenu (session *sess, GdkEventButton *event, chan *ch)
 {
 	GtkWidget *menu, *item;
-	session *sess = ud;
 	char buf[256];
 
-	/* shift-click to close a tab */
-	if ((event->state & GDK_SHIFT_MASK) && event->type == GDK_BUTTON_PRESS)
-	{
-		mg_xbutton_cb (cv, ch, tag, ud);
-		return FALSE;
-	}
-
-	if (event->button != 3)
-		return FALSE;
-
 	menu = gtk_menu_new ();
 
-	if (tag == TAG_IRC)
+	if (sess)
 	{
 		char *name = g_markup_escape_text (sess->channel[0] ? sess->channel : _("<none>"), -1);
 		snprintf (buf, sizeof (buf), "<span foreground=\"#3344cc\"><b>%s</b></span>", name);
 		g_free (name);
+
 		item = gtk_menu_item_new_with_label ("");
 		gtk_label_set_markup (GTK_LABEL (GTK_BIN (item)->child), buf);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 
 		/* separator */
-		item = gtk_menu_item_new ();
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show (item);
+		menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);
+
+		/* per-channel alerts */
+		mg_create_alertmenu (sess, menu);
+
+		/* per-channel settings */
+		mg_create_perchannelmenu (sess, menu);
+
+		/* separator */
+		menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);
 
-		menu_toggle_item (_("Beep on message"), menu, mg_beepmsg_cb, sess,
-								sess->beep);
-		if (prefs.gui_tray)
-			menu_toggle_item (_("Blink tray on message"), menu, mg_traymsg_cb, sess,
-									sess->tray);
 		if (sess->type == SESS_CHANNEL)
-			menu_toggle_item (_("Show join/part messages"), menu, mg_hidejp_cb,
-									sess, !sess->hide_join_part);
-		menu_toggle_item (_("Color paste"), menu, mg_colorpaste_cb, sess,
-								sess->color_paste);
-
+			menu_addfavoritemenu (sess->server, menu, sess->channel);
 	}
 
-	/* separator */
-	item = gtk_menu_item_new ();
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	mg_create_icon_item (_("_Close Tab"), GTK_STOCK_CLOSE, menu,
-								mg_destroy_tab_cb, ch);
-	mg_create_icon_item (_("_Detach Tab"), GTK_STOCK_REDO, menu,
+	mg_create_icon_item (_("_Detach"), GTK_STOCK_REDO, menu,
 								mg_detach_tab_cb, ch);
-
+	mg_create_icon_item (_("_Close"), GTK_STOCK_CLOSE, menu,
+								mg_destroy_tab_cb, ch);
 	if (sess && tabmenu_list)
 		menu_create (menu, tabmenu_list, sess->channel, FALSE);
 	menu_add_plugin_items (menu, "\x4$TAB", sess->channel);
@@ -1661,6 +1673,26 @@ mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventBut
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 							G_CALLBACK (mg_menu_destroy), NULL);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
+}
+
+static gboolean
+mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventButton *event)
+{
+	/* shift-click to close a tab */
+	if ((event->state & GDK_SHIFT_MASK) && event->type == GDK_BUTTON_PRESS)
+	{
+		mg_xbutton_cb (cv, ch, tag, ud);
+		return FALSE;
+	}
+
+	if (event->button != 3)
+		return FALSE;
+
+	if (tag == TAG_IRC)
+		mg_create_tabmenu (ud, event, ch);
+	else
+		mg_create_tabmenu (NULL, event, ch);
+
 	return TRUE;
 }
 
@@ -2670,6 +2702,9 @@ mg_place_userlist_and_chanview_real (session_gui *gui, GtkWidget *userlist, GtkW
 
 	if (chanview)
 	{
+		/* incase the previous pos was POS_HIDDEN */
+		gtk_widget_show (chanview);
+
 		gtk_table_set_row_spacing (GTK_TABLE (gui->main_table), 1, 0);
 		gtk_table_set_row_spacing (GTK_TABLE (gui->main_table), 2, 2);
 
@@ -2694,6 +2729,15 @@ mg_place_userlist_and_chanview_real (session_gui *gui, GtkWidget *userlist, GtkW
 									1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 			break;
 		case POS_HIDDEN:
+			gtk_widget_hide (chanview);
+			/* always attach it to something to avoid ref_count=0 */
+			if (prefs.gui_ulist_pos == POS_TOP)
+				gtk_table_attach (GTK_TABLE (gui->main_table), chanview,
+										1, 2, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
+
+			else
+				gtk_table_attach (GTK_TABLE (gui->main_table), chanview,
+										1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 			break;
 		default:/* POS_BOTTOM */
 			gtk_table_set_row_spacing (GTK_TABLE (gui->main_table), 2, 3);
@@ -2715,8 +2759,8 @@ mg_place_userlist_and_chanview_real (session_gui *gui, GtkWidget *userlist, GtkW
 		case POS_BOTTOMRIGHT:
 			gtk_paned_pack2 (GTK_PANED (gui->vpane_right), userlist, FALSE, TRUE);
 			break;
-		case POS_HIDDEN:
-			break;
+		/*case POS_HIDDEN:
+			break;*/	/* Hide using the VIEW menu instead */
 		default:/* POS_TOPRIGHT */
 			gtk_paned_pack1 (GTK_PANED (gui->vpane_right), userlist, FALSE, TRUE);
 		}
@@ -2882,8 +2926,15 @@ mg_tabs_compare (session *a, session *b)
 static void
 mg_create_tabs (session_gui *gui)
 {
+	gboolean use_icons = FALSE;
+
+	/* if any one of these PNGs exist, the chanview will create
+	 * the extra column for icons. */
+	if (pix_channel || pix_dialog || pix_server || pix_util)
+		use_icons = TRUE;
+
 	gui->chanview = chanview_new (prefs.tab_layout, prefs.truncchans,
-											prefs.tab_sort, prefs.tab_icons,
+											prefs.tab_sort, use_icons,
 											prefs.style_namelistgad ? input_style : NULL);
 	chanview_set_callbacks (gui->chanview, mg_switch_tab_cb, mg_xbutton_cb,
 									mg_tab_contextmenu_cb, (void *)mg_tabs_compare);
@@ -3315,13 +3366,6 @@ fe_set_away (server *serv)
 	}
 }
 
-void
-fe_set_color_paste (session *sess, int status)
-{
-	sess->color_paste = status;
-	if (!sess->gui->is_tab || sess == current_tab)
-		GTK_XTEXT (sess->gui->xtext)->color_paste = status;
-}
 void
 fe_set_channel (session *sess)
 {
@@ -3359,7 +3403,7 @@ mg_changui_new (session *sess, restore_gui *res, int tab, int focus)
 		mg_create_topwindow (sess);
 		fe_set_title (sess);
 		if (user && user->hostname)
-			set_topic (sess, user->hostname);
+			set_topic (sess, user->hostname, user->hostname);
 		return;
 	}
 
@@ -3380,7 +3424,7 @@ mg_changui_new (session *sess, restore_gui *res, int tab, int focus)
 	}
 
 	if (user && user->hostname)
-		set_topic (sess, user->hostname);
+		set_topic (sess, user->hostname, user->hostname);
 
 	mg_add_chan (sess);
 
@@ -3579,6 +3623,30 @@ mg_handle_drop (GtkWidget *widget, int y, int *pos, int *other_pos)
 	mg_place_userlist_and_chanview (gui);
 }
 
+static gboolean
+mg_is_gui_target (GdkDragContext *context)
+{
+	char *target_name;
+
+	if (!context || !context->targets || !context->targets->data)
+		return FALSE;
+
+	target_name = gdk_atom_name (context->targets->data);
+	if (target_name)
+	{
+		/* if it's not XCHAT_CHANVIEW or XCHAT_USERLIST */
+		/* we should ignore it. */
+		if (target_name[0] != 'X')
+		{
+			g_free (target_name);
+			return FALSE;
+		}
+		g_free (target_name);
+	}
+
+	return TRUE;
+}
+
 /* this begin callback just creates an nice of the source */
 
 gboolean
@@ -3589,6 +3657,10 @@ mg_drag_begin_cb (GtkWidget *widget, GdkDragContext *context, gpointer userdata)
 	GdkColormap *cmap;
 	GdkPixbuf *pix, *pix2;
 
+	/* ignore file drops */
+	if (!mg_is_gui_target (context))
+		return FALSE;
+
 	cmap = gtk_widget_get_colormap (widget);
 	gdk_drawable_get_size (widget->window, &width, &height);
 
@@ -3606,6 +3678,10 @@ mg_drag_begin_cb (GtkWidget *widget, GdkDragContext *context, gpointer userdata)
 void
 mg_drag_end_cb (GtkWidget *widget, GdkDragContext *context, gpointer userdata)
 {
+	/* ignore file drops */
+	if (!mg_is_gui_target (context))
+		return;
+
 #ifndef WIN32
 	g_object_unref (g_object_get_data (G_OBJECT (widget), "ico"));
 #endif
@@ -3616,6 +3692,10 @@ mg_drag_end_cb (GtkWidget *widget, GdkDragContext *context, gpointer userdata)
 gboolean
 mg_drag_drop_cb (GtkWidget *widget, GdkDragContext *context, int x, int y, guint time, gpointer user_data)
 {
+	/* ignore file drops */
+	if (!mg_is_gui_target (context))
+		return FALSE;
+
 	switch (context->action)
 	{
 	case GDK_ACTION_MOVE:
@@ -3646,6 +3726,10 @@ mg_drag_motion_cb (GtkWidget *widget, GdkDragContext *context, int x, int y, gui
 	GtkPaned *paned;
 	GdkDrawable *draw;
 
+	/* ignore file drops */
+	if (!mg_is_gui_target (context))
+		return FALSE;
+
 	if (scbar)	/* scrollbar */
 	{
 		ox = widget->allocation.x;
