@@ -1,5 +1,5 @@
 /* X-Chat
- * Copyright (C) 1998 Peter Zelezny.
+ * Copyright (C) 1998-2007 Peter Zelezny.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,6 +51,8 @@
 #include "../common/ignore.h"
 #include "../common/fe.h"
 #include "../common/server.h"
+#include "../common/servlist.h"
+#include "../common/notify.h"
 #include "../common/util.h"
 #include "xtext.h"
 #include "about.h"
@@ -251,7 +253,7 @@ menu_toggle_item (char *label, GtkWidget *menu, void *callback, void *userdata,
 {
 	GtkWidget *item;
 
-	item = gtk_check_menu_item_new_with_label (label);
+	item = gtk_check_menu_item_new_with_mnemonic (label);
 	gtk_check_menu_item_set_active ((GtkCheckMenuItem*)item, state);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate",
@@ -261,11 +263,12 @@ menu_toggle_item (char *label, GtkWidget *menu, void *callback, void *userdata,
 	return item;
 }
 
-static GtkWidget *
+GtkWidget *
 menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags,
 					  gpointer userdata, char *icon)
 {
 	GtkWidget *img, *item;
+	char *path;
 
 	if (!label)
 		item = gtk_menu_item_new ();
@@ -277,22 +280,32 @@ menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags,
 				item = gtk_image_menu_item_new_with_markup (label);
 			else*/
 				item = gtk_image_menu_item_new_with_mnemonic (label);
-			img = gtk_image_new_from_file (icon);
-			if (img)
-				gtk_image_menu_item_set_image ((GtkImageMenuItem *)item, img);
+			img = NULL;
+			if (access (icon, R_OK) == 0)	/* try fullpath */
+				img = gtk_image_new_from_file (icon);
 			else
 			{
-				img = gtk_image_new_from_stock (icon, GTK_ICON_SIZE_MENU);
-				if (img)
-					gtk_image_menu_item_set_image ((GtkImageMenuItem *)item, img);
+				/* try relative to ~/.xchat2 */
+				path = g_strdup_printf ("%s/%s", get_xdir_fs (), icon);
+				if (access (path, R_OK) == 0)
+					img = gtk_image_new_from_file (path);
+				else
+					img = gtk_image_new_from_stock (icon, GTK_ICON_SIZE_MENU);
+				g_free (path);
 			}
+
+			if (img)
+				gtk_image_menu_item_set_image ((GtkImageMenuItem *)item, img);
 		}
 		else
 		{
 			if (flags & XCMENU_MARKUP)
 			{
 				item = gtk_menu_item_new_with_label ("");
-				gtk_label_set_markup_with_mnemonic (GTK_LABEL (GTK_BIN (item)->child), label);
+				if (flags & XCMENU_MNEMONIC)
+					gtk_label_set_markup_with_mnemonic (GTK_LABEL (GTK_BIN (item)->child), label);
+				else
+					gtk_label_set_markup (GTK_LABEL (GTK_BIN (item)->child), label);
 			} else
 			{
 				if (flags & XCMENU_MNEMONIC)
@@ -327,7 +340,7 @@ menu_quick_item_with_callback (void *callback, char *label, GtkWidget * menu,
 	gtk_widget_show (item);
 }
 
-static GtkWidget *
+GtkWidget *
 menu_quick_sub (char *name, GtkWidget *menu, GtkWidget **sub_item_ret, int flags, int pos)
 {
 	GtkWidget *sub_menu;
@@ -418,6 +431,46 @@ is_in_path (char *cmd)
 	return 0;
 }
 
+/* syntax: "LABEL~ICON~STUFF~ADDED~LATER~" */
+
+static void
+menu_extract_icon (char *name, char **label, char **icon)
+{
+	char *p = name;
+	char *start = NULL;
+	char *end = NULL;
+
+	while (*p)
+	{
+		if (*p == '~')
+		{
+			/* escape \~ */
+			if (p == name || p[-1] != '\\')
+			{
+				if (!start)
+					start = p + 1;
+				else if (!end)
+					end = p + 1;
+			}
+		}
+		p++;
+	}
+
+	if (!end)
+		end = p;
+
+	if (start && start != end)
+	{
+		*label = g_strndup (name, (start - name) - 1);
+		*icon = g_strndup (start, (end - start) - 1);
+	}
+	else
+	{
+		*label = g_strdup (name);
+		*icon = NULL;
+	}
+}
+
 /* append items to "menu" using the (struct popup*) list provided */
 
 void
@@ -435,7 +488,7 @@ menu_create (GtkWidget *menu, GSList *list, char *target, int check_path)
 		if (!strncasecmp (pop->name, "SUB", 3))
 		{
 			childcount = 0;
-			tempmenu = menu_quick_sub (pop->cmd, tempmenu, &subitem, XCMENU_DOLIST, -1);
+			tempmenu = menu_quick_sub (pop->cmd, tempmenu, &subitem, XCMENU_DOLIST|XCMENU_MNEMONIC, -1);
 
 		} else if (!strncasecmp (pop->name, "TOGGLE", 6))
 		{
@@ -460,15 +513,33 @@ menu_create (GtkWidget *menu, GSList *list, char *target, int check_path)
 
 		} else
 		{
+			char *icon, *label;
+
+			/* default command in xchat.c */
+			if (pop->cmd[0] == 'n' && !strcmp (pop->cmd, "notify -n ASK %s"))
+			{
+				/* don't create this item if already in notify list */
+				if (!target || notify_is_in_list (current_sess->server, target))
+				{
+					list = list->next;
+					continue;
+				}
+			}
+
+			menu_extract_icon (pop->name, &label, &icon);
+
 			if (!check_path || pop->cmd[0] != '!')
 			{
-				menu_quick_item (pop->cmd, pop->name, tempmenu, 0, target, 0);
+				menu_quick_item (pop->cmd, label, tempmenu, 0, target, icon);
 			/* check if the program is in path, if not, leave it out! */
 			} else if (is_in_path (pop->cmd))
 			{
 				childcount++;
-				menu_quick_item (pop->cmd, pop->name, tempmenu, 0, target, 0);
+				menu_quick_item (pop->cmd, label, tempmenu, 0, target, icon);
 			}
+
+			g_free (label);
+			g_free (icon);
 		}
 
 		list = list->next;
@@ -479,6 +550,9 @@ menu_create (GtkWidget *menu, GSList *list, char *target, int check_path)
 		submenu_list = g_slist_remove (submenu_list, submenu_list->data);
 }
 
+static char *str_copy = NULL;		/* for all pop-up menus */
+static GtkWidget *nick_submenu = NULL;	/* user info submenu */
+
 static void
 menu_destroy (GtkWidget *menu, gpointer objtounref)
 {
@@ -486,6 +560,7 @@ menu_destroy (GtkWidget *menu, gpointer objtounref)
 	g_object_unref (menu);
 	if (objtounref)
 		g_object_unref (G_OBJECT (objtounref));
+	nick_submenu = NULL;
 }
 
 static void
@@ -505,15 +580,122 @@ menu_popup (GtkWidget *menu, GdkEventButton *event, gpointer objtounref)
 						 0, event ? event->time : 0);
 }
 
-static char *str_copy = 0;		/* for all pop-up menus */
+static void
+menu_nickinfo_cb (GtkWidget *menu, session *sess)
+{
+	char buf[512];
+
+	if (!is_session (sess))
+		return;
+
+	/* issue a /WHOIS */
+	snprintf (buf, sizeof (buf), "WHOIS %s %s", str_copy, str_copy);
+	handle_command (sess, buf, FALSE);
+	/* and hide the output */
+	sess->server->skip_next_whois = 1;
+}
+
+/* returns boolean: Some data is missing */
+
+static gboolean
+menu_create_nickinfo_menu (struct User *user, GtkWidget *submenu)
+{
+	char buf[512];
+	char unknown[96];
+	char *real, *fmt;
+	struct away_msg *away;
+	gboolean missing = FALSE;
+
+	/* let the translators tweak this if need be */
+	fmt = _("<tt><b>%-11s</b></tt> %s");
+	snprintf (unknown, sizeof (unknown), "<i>%s</i>", _("Unknown"));
+
+	if (user->realname)
+	{
+		real = strip_color (user->realname, -1, STRIP_ALL|STRIP_ESCMARKUP);
+		snprintf (buf, sizeof (buf), fmt, _("Real Name:"), real);
+		g_free (real);
+	} else
+	{
+		snprintf (buf, sizeof (buf), fmt, _("Real Name:"), unknown);
+	}
+	menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
+
+	snprintf (buf, sizeof (buf), fmt, _("User:"),
+				 user->hostname ? user->hostname : unknown);
+	menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
+
+	snprintf (buf, sizeof (buf), fmt, _("Country:"),
+				 user->hostname ? country(user->hostname) : unknown);
+	menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
+
+	snprintf (buf, sizeof (buf), fmt, _("Server:"),
+				 user->servername ? user->servername : unknown);
+	menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
+
+	if (user->lasttalk)
+	{
+		char min[96];
+
+		snprintf (min, sizeof (min), _("%u minutes ago"),
+					(unsigned int) ((time (0) - user->lasttalk) / 60));
+		snprintf (buf, sizeof (buf), fmt, _("Last Msg:"), min);
+	} else
+	{
+		snprintf (buf, sizeof (buf), fmt, _("Last Msg:"), unknown);
+	}
+	menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
+
+	if (user->away)
+	{
+		away = server_away_find_message (current_sess->server, user->nick);
+		if (away)
+		{
+			char *msg = strip_color (away->message ? away->message : unknown, -1, STRIP_ALL|STRIP_ESCMARKUP);
+			snprintf (buf, sizeof (buf), fmt, _("Away Msg:"), msg);
+			g_free (msg);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
+		}
+		else
+			missing = TRUE;
+	}
+
+	return missing;
+}
+
+void
+fe_userlist_update (session *sess, struct User *user)
+{
+	GList *items, *next;
+
+	if (!nick_submenu || !str_copy)
+		return;
+
+	/* not the same nick as the menu? */
+	if (sess->server->p_cmp (user->nick, str_copy))
+		return;
+
+	/* get rid of the "show" signal */
+	g_signal_handlers_disconnect_by_func (nick_submenu, menu_nickinfo_cb, sess);
+
+	/* destroy all the old items */
+	items = ((GtkMenuShell *) nick_submenu)->children;
+	while (items)
+	{
+		next = items->next;
+		gtk_widget_destroy (items->data);
+		items = next;
+	}
+
+	/* and re-create them with new info */
+	menu_create_nickinfo_menu (user, nick_submenu);
+}
 
 void
 menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 {
 	char buf[512];
-	char *real, *fmt;
 	struct User *user;
-	struct away_msg *away;
 	GtkWidget *submenu, *menu = gtk_menu_new ();
 
 	if (str_copy)
@@ -525,7 +707,7 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 	/* more than 1 nick selected? */
 	if (num_sel > 1)
 	{
-		snprintf (buf, sizeof (buf), "%d nicks selected.", num_sel);
+		snprintf (buf, sizeof (buf), _("%d nicks selected."), num_sel);
 		menu_quick_item (0, buf, menu, 0, 0, 0);
 		menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);
 	} else
@@ -535,61 +717,13 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 			user = userlist_find_global (current_sess->server, nick);
 		if (user)
 		{
-			submenu = menu_quick_sub (nick, menu, NULL, XCMENU_DOLIST, -1);
+			nick_submenu = submenu = menu_quick_sub (nick, menu, NULL, XCMENU_DOLIST, -1);
 
-			/* let the translators tweak this if need be */
-			fmt = _("<tt><b>%-11s</b></tt> %s");
-
-			if (user->realname)
+			if (menu_create_nickinfo_menu (user, submenu) ||
+				 !user->hostname || !user->realname || !user->servername)
 			{
-				real = g_markup_escape_text (user->realname, -1);
-				snprintf (buf, sizeof (buf), fmt, _("Real Name:"), real);
-				g_free (real);
-			} else
-			{
-				snprintf (buf, sizeof (buf), fmt, _("Real Name:"), _("Unknown"));
+				g_signal_connect (G_OBJECT (submenu), "show", G_CALLBACK (menu_nickinfo_cb), sess);
 			}
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
-
-			snprintf (buf, sizeof (buf), fmt, _("User:"),
-						user->hostname ? user->hostname : _("Unknown"));
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
-
-			snprintf (buf, sizeof (buf), fmt, _("Country:"),
-						user->hostname ? country(user->hostname) : _("Unknown"));
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
-
-			snprintf (buf, sizeof (buf), fmt, _("Server:"),
-						user->servername ? user->servername : _("Unknown"));
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
-
-			if (user->away)
-			{
-				away = server_away_find_message (current_sess->server, nick);
-				if (away)
-				{
-					char *msg = strip_color (away->message ? away->message : _("Unknown"), -1, STRIP_ALL);
-					real = g_markup_escape_text (msg, -1);
-					free (msg);
-					snprintf (buf, sizeof (buf), fmt, _("Away Msg:"), real);
-					g_free (real);
-					menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
-				}
-			}
-
-			if (user->lasttalk)
-			{
-				char min[96];
-
-				snprintf (min, sizeof (min), _("%u minutes ago"),
-							(unsigned int) ((time (0) - user->lasttalk) / 60));
-				snprintf (buf, sizeof (buf), fmt, _("Last Msg:"), min);
-			} else
-			{
-				snprintf (buf, sizeof (buf), fmt, _("Last Msg:"), _("Unknown"));
-			}
-
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
 
 			menu_quick_endsub ();
 			menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);
@@ -600,7 +734,12 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 		menu_create (menu, popup_list, NULL, FALSE);
 	else
 		menu_create (menu, popup_list, str_copy, FALSE);
-	menu_add_plugin_items (menu, "\x5$NICK");
+
+	if (num_sel == 0)	/* xtext click */
+		menu_add_plugin_items (menu, "\x5$NICK", str_copy);
+	else	/* userlist treeview click */
+		menu_add_plugin_items (menu, "\x5$NICK", NULL);
+
 	menu_popup (menu, event, NULL);
 }
 
@@ -624,6 +763,12 @@ menu_topic_showhide_cb (session *sess)
 		gtk_widget_hide (sess->gui->topic_bar);
 }
 
+static void
+menu_userlist_showhide_cb (session *sess)
+{
+	mg_decide_userlist (sess, TRUE);
+}
+
 static void
 menu_ulbuttons_showhide_cb (session *sess)
 {
@@ -700,6 +845,14 @@ menu_topicbar_toggle (GtkWidget *wid, gpointer ud)
 								 prefs.topicbar);
 }
 
+static void
+menu_userlist_toggle (GtkWidget *wid, gpointer ud)
+{
+	prefs.hideuserlist = !prefs.hideuserlist;
+	menu_setting_foreach (menu_userlist_showhide_cb, MENU_ID_USERLIST,
+								 !prefs.hideuserlist);
+}
+
 static void
 menu_ulbuttons_toggle (GtkWidget *wid, gpointer ud)
 {
@@ -778,15 +931,20 @@ menu_urlmenu (GdkEventButton *event, char *url)
 	menu_quick_item_with_callback (copy_to_clipboard_cb, _("Copy Selected Link"), menu, str_copy);
 	/* custom ones from urlhandlers.conf */
 	menu_create (menu, urlhandler_list, str_copy, TRUE);
-	menu_add_plugin_items (menu, "\x4$URL");
+	menu_add_plugin_items (menu, "\x4$URL", str_copy);
 	menu_popup (menu, event, NULL);
 }
 
 static void
 menu_chan_cycle (GtkWidget * menu, char *chan)
 {
+	char tbuf[256];
+
 	if (current_sess)
-		handle_command (current_sess, "CYCLE", FALSE);
+	{
+		snprintf (tbuf, sizeof tbuf, "CYCLE %s", chan);
+		handle_command (current_sess, tbuf, FALSE);
+	}
 }
 
 static void
@@ -842,9 +1000,43 @@ menu_chanmenu (struct session *sess, GdkEventButton * event, char *chan)
 												 str_copy);
 	}
 
+	menu_addfavoritemenu (sess->server, menu, str_copy);
+
+	menu_add_plugin_items (menu, "\x5$CHAN", str_copy);
 	menu_popup (menu, event, NULL);
 }
 
+static void
+menu_delfav_cb (GtkWidget *item, server *serv)
+{
+	servlist_autojoinedit (serv->network, str_copy, FALSE);
+}
+
+static void
+menu_addfav_cb (GtkWidget *item, server *serv)
+{
+	servlist_autojoinedit (serv->network, str_copy, TRUE);
+}
+
+void
+menu_addfavoritemenu (server *serv, GtkWidget *menu, char *channel)
+{
+	if (!serv->network)
+		return;
+
+	if (channel != str_copy)
+	{
+		if (str_copy)
+			free (str_copy);
+		str_copy = strdup (channel);
+	}
+
+	if (joinlist_is_in_list (serv, channel))
+		mg_create_icon_item (_("_Remove from Favorites"), GTK_STOCK_REMOVE, menu, menu_delfav_cb, serv);
+	else
+		mg_create_icon_item (_("_Add to Favorites"), GTK_STOCK_ADD, menu, menu_addfav_cb, serv);
+}
+
 static void
 menu_open_server_list (GtkWidget *wid, gpointer none)
 {
@@ -979,6 +1171,12 @@ menu_close (GtkWidget * wid, gpointer none)
 	mg_close_sess (current_sess);
 }
 
+static void
+menu_quit (GtkWidget * wid, gpointer none)
+{
+	mg_open_quit_dialog (FALSE);
+}
+
 static void
 menu_search ()
 {
@@ -994,7 +1192,7 @@ menu_resetmarker (GtkWidget * wid, gpointer none)
 static void
 menu_flushbuffer (GtkWidget * wid, gpointer none)
 {
-	fe_text_clear (current_sess);
+	fe_text_clear (current_sess, 0);
 }
 
 static void
@@ -1272,24 +1470,32 @@ menu_dcc_chat_win (GtkWidget *wid, gpointer none)
 	fe_dcc_open_chat_win (FALSE);
 }
 
-static void
-menu_layout_cb (GtkWidget *item, gpointer none)
+void
+menu_change_layout (void)
 {
-	if (GTK_CHECK_MENU_ITEM (item)->active)
+	if (prefs.tab_layout == 0)
 	{
-		prefs.tab_layout = 0;
 		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TABS, 1);
 		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TREE, 0);
 		mg_change_layout (0);
 	} else
 	{
-		prefs.tab_layout = 2;
 		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TABS, 0);
 		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TREE, 1);
 		mg_change_layout (2);
 	}
 }
 
+static void
+menu_layout_cb (GtkWidget *item, gpointer none)
+{
+	prefs.tab_layout = 2;
+	if (GTK_CHECK_MENU_ITEM (item)->active)
+		prefs.tab_layout = 0;
+
+	menu_change_layout ();
+}
+
 static void
 menu_apply_metres_cb (session *sess)
 {
@@ -1364,37 +1570,39 @@ static struct mymenu mymenu[] = {
 #define CLOSE_OFFSET (13)
 	{0, menu_close, GTK_STOCK_CLOSE, M_MENUSTOCK, 0, 0, 1, GDK_w},
 	{0, 0, 0, M_SEP, 0, 0, 0},
-	{N_("_Quit"), mg_safe_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 0, 1, GDK_q},	/* 15 */
+	{N_("_Quit"), menu_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 0, 1, GDK_q},	/* 15 */
 
 	{N_("_View"), 0, 0, M_NEWMENU, 0, 0, 1},
 #define MENUBAR_OFFSET (17)
-	{N_("_Menubar"), menu_bar_toggle_cb, 0, M_MENUTOG, MENU_ID_MENUBAR, 0, 1, GDK_F9},
-	{N_("_Topicbar"), menu_topicbar_toggle, 0, M_MENUTOG, MENU_ID_TOPICBAR, 0, 1},
-	{N_("_Userlist Buttons"), menu_ulbuttons_toggle, 0, M_MENUTOG, MENU_ID_ULBUTTONS, 0, 1},
+	{N_("_Menu Bar"), menu_bar_toggle_cb, 0, M_MENUTOG, MENU_ID_MENUBAR, 0, 1, GDK_F9},
+	{N_("_Topic Bar"), menu_topicbar_toggle, 0, M_MENUTOG, MENU_ID_TOPICBAR, 0, 1},
+	{N_("_User List"), menu_userlist_toggle, 0, M_MENUTOG, MENU_ID_USERLIST, 0, 1, GDK_F7},
+	{N_("U_serlist Buttons"), menu_ulbuttons_toggle, 0, M_MENUTOG, MENU_ID_ULBUTTONS, 0, 1},
 	{N_("M_ode Buttons"), menu_cmbuttons_toggle, 0, M_MENUTOG, MENU_ID_MODEBUTTONS, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0, 0},
-	{N_("_Layout"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 22 */
-#define TABS_OFFSET (23)
+	{N_("_Channel Switcher"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 23 */
+#define TABS_OFFSET (24)
 		{N_("_Tabs"), menu_layout_cb, 0, M_MENURADIO, MENU_ID_LAYOUT_TABS, 0, 1},
 		{N_("T_ree"), 0, 0, M_MENURADIO, MENU_ID_LAYOUT_TREE, 0, 1},
 		{0, 0, 0, M_END, 0, 0, 0},
-	{N_("_Network Meters"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 26 */
-#define METRE_OFFSET (27)
+	{N_("_Network Meters"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 27 */
+#define METRE_OFFSET (28)
 		{N_("Off"), menu_metres_off, 0, M_MENURADIO, 0, 0, 1},
 		{N_("Graph"), menu_metres_graph, 0, M_MENURADIO, 0, 0, 1},
 		{N_("Text"), menu_metres_text, 0, M_MENURADIO, 0, 0, 1},
 		{N_("Both"), menu_metres_both, 0, M_MENURADIO, 0, 0, 1},
-		{0, 0, 0, M_END, 0, 0, 0},	/* 31 */
+		{0, 0, 0, M_END, 0, 0, 0},	/* 32 */
 
 	{N_("_Server"), 0, 0, M_NEWMENU, 0, 0, 1},
 	{N_("_Disconnect"), menu_disconnect, GTK_STOCK_DISCONNECT, M_MENUSTOCK, MENU_ID_DISCONNECT, 0, 1},
 	{N_("_Reconnect"), menu_reconnect, GTK_STOCK_CONNECT, M_MENUSTOCK, MENU_ID_RECONNECT, 0, 1},
-	{N_("Join Channel..."), menu_join, GTK_STOCK_JUMP_TO, M_MENUSTOCK, MENU_ID_JOIN, 0, 1},
+	{N_("Join a Channel..."), menu_join, GTK_STOCK_JUMP_TO, M_MENUSTOCK, MENU_ID_JOIN, 0, 1},
+	{N_("List of Channels..."), menu_chanlist, GTK_STOCK_INDEX, M_MENUITEM, 0, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0, 0},
-#define AWAY_OFFSET (37)
+#define AWAY_OFFSET (39)
 	{N_("Marked Away"), menu_away, 0, M_MENUTOG, MENU_ID_AWAY, 0, 1, GDK_a},
 
-	{N_("_Usermenu"), 0, 0, M_NEWMENU, MENU_ID_USERMENU, 0, 1},	/* 38 */
+	{N_("_Usermenu"), 0, 0, M_NEWMENU, MENU_ID_USERMENU, 0, 1},	/* 40 */
 
 	{N_("S_ettings"), 0, 0, M_NEWMENU, 0, 0, 1},
 	{N_("_Preferences"), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 0, 1},
@@ -1409,27 +1617,30 @@ static struct mymenu mymenu[] = {
 		{N_("User Commands..."), menu_usercommands, 0, M_MENUITEM, 0, 0, 1},
 		{N_("Userlist Buttons..."), menu_ulbuttons, 0, M_MENUITEM, 0, 0, 1},
 		{N_("Userlist Popup..."), menu_ulpopup, 0, M_MENUITEM, 0, 0, 1},
-		{0, 0, 0, M_END, 0, 0, 0},		/* 51 */
+		{0, 0, 0, M_END, 0, 0, 0},		/* 53 */
 
 	{N_("_Window"), 0, 0, M_NEWMENU, 0, 0, 1},
 	{N_("Ban List..."), menu_banlist, 0, M_MENUITEM, 0, 0, 1},
-	{N_("Channel List..."), menu_chanlist, 0, M_MENUITEM, 0, 0, 1},
 	{N_("Character Chart..."), ascii_open, 0, M_MENUITEM, 0, 0, 1},
 	{N_("Direct Chat..."), menu_dcc_chat_win, 0, M_MENUITEM, 0, 0, 1},
 	{N_("File Transfers..."), menu_dcc_win, 0, M_MENUITEM, 0, 0, 1},
+	{N_("Friends List..."), notify_opengui, 0, M_MENUITEM, 0, 0, 1},
 	{N_("Ignore List..."), ignore_gui_open, 0, M_MENUITEM, 0, 0, 1},
-	{N_("Notify List..."), notify_opengui, 0, M_MENUITEM, 0, 0, 1},
 	{N_("Plugins and Scripts..."), menu_pluginlist, 0, M_MENUITEM, 0, 0, 1},
-	{N_("Raw Log..."), menu_rawlog, 0, M_MENUITEM, 0, 0, 1},	/* 61 */
+	{N_("Raw Log..."), menu_rawlog, 0, M_MENUITEM, 0, 0, 1},	/* 62 */
 	{N_("URL Grabber..."), url_opengui, 0, M_MENUITEM, 0, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0, 0},
 	{N_("Reset Marker Line"), menu_resetmarker, 0, M_MENUITEM, 0, 0, 1, GDK_m},
 	{N_("C_lear Text"), menu_flushbuffer, GTK_STOCK_CLEAR, M_MENUSTOCK, 0, 0, 1, GDK_l},
+#define SEARCH_OFFSET 67
 	{N_("Search Text..."), menu_search, GTK_STOCK_FIND, M_MENUSTOCK, 0, 0, 1, GDK_f},
 	{N_("Save Text..."), menu_savebuffer, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 0, 1},
 
-	{N_("_Help"), 0, 0, M_NEWMENU, 0, 0, 1},	/* 68 */
+	{N_("_Help"), 0, 0, M_NEWMENU, 0, 0, 1},	/* 69 */
 	{N_("_Contents"), menu_docs, GTK_STOCK_HELP, M_MENUSTOCK, 0, 0, 1, GDK_F1},
+#if 0
+	{N_("Check for updates"), menu_update, 0, M_MENUITEM, 0, 1},
+#endif
 	{N_("_About"), menu_about, GTK_STOCK_ABOUT, M_MENUSTOCK, 0, 0, 1},
 
 	{0, 0, 0, M_END, 0, 0, 0},
@@ -1541,7 +1752,7 @@ menu_find (GtkWidget *menu, char *path, char *label)
 }
 
 static void
-menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *))
+menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *, char *))
 {
 	GSList *list = sess_list;
 	int tabdone = FALSE;
@@ -1556,7 +1767,7 @@ menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *))
 		/* do it only once for tab sessions, since they share a GUI */
 		if (!sess->gui->is_tab || !tabdone)
 		{
-			callback (sess->gui->menu, me);
+			callback (sess->gui->menu, me, NULL);
 			if (sess->gui->is_tab)
 				tabdone = TRUE;
 		}
@@ -1565,7 +1776,7 @@ menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *))
 }
 
 static void
-menu_update_cb (GtkWidget *menu, menu_entry *me)
+menu_update_cb (GtkWidget *menu, menu_entry *me, char *target)
 {
 	GtkWidget *item;
 
@@ -1680,7 +1891,7 @@ menu_add_toggle (GtkWidget *menu, menu_entry *me)
 }
 
 static GtkWidget *
-menu_add_item (GtkWidget *menu, menu_entry *me)
+menu_add_item (GtkWidget *menu, menu_entry *me, char *target)
 {
 	GtkWidget *item = NULL;
 	char *path = me->path + me->root_offset;
@@ -1689,7 +1900,7 @@ menu_add_item (GtkWidget *menu, menu_entry *me)
 		menu = menu_find_path (menu, path);
 	if (menu)
 	{
-		item = menu_quick_item (me->cmd, me->label, menu, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, 0, me->icon);
+		item = menu_quick_item (me->cmd, me->label, menu, me->markup ? XCMENU_MARKUP|XCMENU_MNEMONIC : XCMENU_MNEMONIC, target, me->icon);
 		menu_reorder (GTK_MENU (menu), item, me->pos);
 	}
 	return item;
@@ -1709,13 +1920,13 @@ menu_add_sub (GtkWidget *menu, menu_entry *me)
 		pos = me->pos;
 		if (pos < 0)	/* position offset from end/bottom */
 			pos = g_list_length (GTK_MENU_SHELL (menu)->children) + pos;
-		menu_quick_sub (me->label, menu, &item, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, pos);
+		menu_quick_sub (me->label, menu, &item, me->markup ? XCMENU_MARKUP|XCMENU_MNEMONIC : XCMENU_MNEMONIC, pos);
 	}
 	return item;
 }
 
 static void
-menu_del_cb (GtkWidget *menu, menu_entry *me)
+menu_del_cb (GtkWidget *menu, menu_entry *me, char *target)
 {
 	GtkWidget *item = menu_find (menu, me->path + me->root_offset, me->label);
 	if (item)
@@ -1723,16 +1934,17 @@ menu_del_cb (GtkWidget *menu, menu_entry *me)
 }
 
 static void
-menu_add_cb (GtkWidget *menu, menu_entry *me)
+menu_add_cb (GtkWidget *menu, menu_entry *me, char *target)
 {
 	GtkWidget *item;
+	GtkAccelGroup *accel_group;
 
 	if (me->group)	/* have a group name? Must be a radio item */
 		item = menu_add_radio (menu, me);
 	else if (me->ucmd)	/* have unselect-cmd? Must be a toggle item */
 		item = menu_add_toggle (menu, me);
 	else if (me->cmd || !me->label)	/* label=NULL for separators */
-		item = menu_add_item (menu, me);
+		item = menu_add_item (menu, me, target);
 	else
 		item = menu_add_sub (menu, me);
 
@@ -1740,9 +1952,12 @@ menu_add_cb (GtkWidget *menu, menu_entry *me)
 	{
 		gtk_widget_set_sensitive (item, me->enable);
 		if (me->key)
-			gtk_widget_add_accelerator (item, "activate",
-											g_object_get_data (G_OBJECT (menu), "accel"),
-											me->key, me->modifier, GTK_ACCEL_VISIBLE);
+		{
+			accel_group = g_object_get_data (G_OBJECT (menu), "accel");
+			if (accel_group)	/* popup menus don't have them */
+				gtk_widget_add_accelerator (item, "activate", accel_group, me->key,
+													 me->modifier, GTK_ACCEL_VISIBLE);
+		}
 	}
 }
 
@@ -1788,13 +2003,13 @@ menu_add_plugin_mainmenu_items (GtkWidget *menu)
 	{
 		me = list->data;
 		if (me->is_main)
-			menu_add_cb (menu, me);
+			menu_add_cb (menu, me, NULL);
 		list = list->next;
 	}
 }
 
 void
-menu_add_plugin_items (GtkWidget *menu, char *root)
+menu_add_plugin_items (GtkWidget *menu, char *root, char *target)
 {
 	GSList *list;
 	menu_entry *me;
@@ -1804,7 +2019,7 @@ menu_add_plugin_items (GtkWidget *menu, char *root)
 	{
 		me = list->data;
 		if (!me->is_main && !strncmp (me->path, root + 1, root[0]))
-			menu_add_cb (menu, me);
+			menu_add_cb (menu, me, target);
 		list = list->next;
 	}
 }
@@ -1844,8 +2059,9 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 	/* set the initial state of toggles */
 	mymenu[MENUBAR_OFFSET].state = !prefs.hidemenu;
 	mymenu[MENUBAR_OFFSET+1].state = prefs.topicbar;
-	mymenu[MENUBAR_OFFSET+2].state = prefs.userlistbuttons;
-	mymenu[MENUBAR_OFFSET+3].state = prefs.chanmodebuttons;
+	mymenu[MENUBAR_OFFSET+2].state = !prefs.hideuserlist;
+	mymenu[MENUBAR_OFFSET+3].state = prefs.userlistbuttons;
+	mymenu[MENUBAR_OFFSET+4].state = prefs.chanmodebuttons;
 
 	mymenu[AWAY_OFFSET].state = away;
 
@@ -1887,7 +2103,10 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 		if (key_theme)
 		{
 			if (!strcasecmp (key_theme, "Emacs"))
+			{
 				close_mask = GDK_SHIFT_MASK | GDK_CONTROL_MASK;
+				mymenu[SEARCH_OFFSET].key = 0;
+			}
 			g_free (key_theme);
 		}
 	}
@@ -1902,13 +2121,13 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 
 	if (!toplevel)
 	{
-		mymenu[DETACH_OFFSET].text = N_("_Detach Tab");
-		mymenu[CLOSE_OFFSET].text = N_("_Close Tab");
+		mymenu[DETACH_OFFSET].text = N_("_Detach");
+		mymenu[CLOSE_OFFSET].text = N_("_Close");
 	}
 	else
 	{
-		mymenu[DETACH_OFFSET].text = N_("_Attach Window");
-		mymenu[CLOSE_OFFSET].text = N_("_Close Window");
+		mymenu[DETACH_OFFSET].text = N_("_Attach");
+		mymenu[CLOSE_OFFSET].text = N_("_Close");
 	}
 
 	while (1)
