@@ -134,10 +134,11 @@ notify_treeview_new (GtkWidget *box)
 
 	view = gtkutil_treeview_new (box, GTK_TREE_MODEL (store),
 	                             notify_treecell_property_mapper,
-	                             USER_COLUMN, _("User"),
+	                             USER_COLUMN, _("Name"),
 	                             STATUS_COLUMN, _("Status"),
 	                             SERVER_COLUMN, _("Network"),
 	                             SEEN_COLUMN, _("Last Seen"), -1);
+	gtk_tree_view_column_set_expand (gtk_tree_view_get_column (GTK_TREE_VIEW (view), 0), TRUE);
 
 	for (col_id=0; (col = gtk_tree_view_get_column (GTK_TREE_VIEW (view), col_id));
 	     col_id++)
@@ -337,8 +338,8 @@ notifygui_add_enter (GtkWidget *entry, GtkWidget *dialog)
 	gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 }
 
-static void
-notify_add_clicked (GtkWidget * igad)
+void
+fe_notify_ask (char *nick, char *networks)
 {
 	GtkWidget *dialog;
 	GtkWidget *entry;
@@ -366,6 +367,7 @@ notify_add_clicked (GtkWidget * igad)
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
 
 	entry = gtk_entry_new ();
+	gtk_entry_set_text (GTK_ENTRY (entry), nick);
 	g_signal_connect (G_OBJECT (entry), "activate",
 						 	G_CALLBACK (notifygui_add_enter), dialog);
 	gtk_table_attach_defaults (GTK_TABLE (table), entry, 1, 2, 0, 1);
@@ -380,7 +382,7 @@ notify_add_clicked (GtkWidget * igad)
 	g_object_set_data (G_OBJECT (entry), "net", wid);
 	g_signal_connect (G_OBJECT (wid), "activate",
 						 	G_CALLBACK (notifygui_add_enter), dialog);
-	gtk_entry_set_text (GTK_ENTRY (wid), "ALL");
+	gtk_entry_set_text (GTK_ENTRY (wid), networks ? networks : "ALL");
 	gtk_table_attach_defaults (GTK_TABLE (table), wid, 1, 2, 2, 3);
 
 	label = gtk_label_new (NULL);
@@ -391,6 +393,12 @@ notify_add_clicked (GtkWidget * igad)
 	gtk_widget_show_all (dialog);
 }
 
+static void
+notify_add_clicked (GtkWidget * igad)
+{
+	fe_notify_ask ("", NULL);
+}
+
 void
 notify_opengui (void)
 {
@@ -404,7 +412,7 @@ notify_opengui (void)
 	}
 
 	notify_window =
-		mg_create_generic_tab ("Notify", _("XChat: Notify List"), FALSE, TRUE,
+		mg_create_generic_tab ("Notify", _("XChat: Friends List"), FALSE, TRUE,
 		                       notify_closegui, NULL, 400, 250, &vbox, 0);
 
 	view = notify_treeview_new (vbox);
