@@ -67,6 +67,7 @@ static int last_selected_row = 0; /* sound row */
 static gboolean color_change;
 static struct xchatprefs setup_prefs;
 static GtkWidget *cancel_button;
+static GtkWidget *font_dialog = NULL;
 
 enum
 {
@@ -250,7 +251,7 @@ static const setting tabs_settings[] =
 	{ST_TOGGLE, N_("Open an extra tab for server notices"), P_OFFINTNL(notices_tabs), 0, 0, 0},
 	{ST_TOGGLE, N_("Open a new tab when you receive a private message"), P_OFFINTNL(autodialog), 0, 0, 0},
 	{ST_TOGGLE, N_("Sort tabs in alphabetical order"), P_OFFINTNL(tab_sort), 0, 0, 0},
-	{ST_TOGGLE, N_("Small tabs"), P_OFFINTNL(tab_small), 0, 0, 0},
+	{ST_TOGGLE, N_("Smaller text"), P_OFFINTNL(tab_small), 0, 0, 0},
 #if 0
 	{ST_MENU,	N_("Focus new tabs:"), P_OFFINTNL(newtabstofront), 0, focusnewtabsmenu, 0},
 #endif
@@ -343,7 +344,7 @@ static const setting alert_settings[] =
 	{ST_ENTRY,	N_("Extra words to highlight:"), P_OFFSETNL(irc_extra_hilight), 0, 0, sizeof prefs.irc_extra_hilight},
 	{ST_ENTRY,	N_("Nick names not to highlight:"), P_OFFSETNL(irc_no_hilight), 0, 0, sizeof prefs.irc_no_hilight},
 	{ST_ENTRY,	N_("Nick names to always highlight:"), P_OFFSETNL(irc_nick_hilight), 0, 0, sizeof prefs.irc_nick_hilight},
-	{ST_LABEL,	N_("Separate multiple words with commas.")},
+	{ST_LABEL,	N_("Separate multiple words with commas.\nWildcards are accepted.")},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
@@ -382,7 +383,8 @@ static const setting advanced_settings[] =
 static const setting logging_settings[] =
 {
 	{ST_HEADER,	N_("Logging"),0,0,0},
-	{ST_TOGGLE,	N_("Enable logging of conversations"), P_OFFINTNL(logging), 0, 0, 2},
+	{ST_TOGGLE,	N_("Display scrollback from previous session"), P_OFFINTNL(text_replay), 0, 0, 0},
+	{ST_TOGGLE,	N_("Enable logging of conversations to disk"), P_OFFINTNL(logging), 0, 0, 2},
 	{ST_ENTRY,	N_("Log filename:"), P_OFFSETNL(logmask), 0, 0, sizeof prefs.logmask},
 	{ST_LABEL,	N_("%s=Server %c=Channel %n=Network.")},
 
@@ -591,6 +593,7 @@ setup_create_italic_label (char *text)
 	label = gtk_label_new (NULL);
 	snprintf (buf, sizeof (buf), "<i><span size=\"smaller\">%s</span></i>", text);
 	gtk_label_set_markup (GTK_LABEL (label), buf);
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
 
 	return label;
 }
@@ -736,7 +739,7 @@ setup_create_radio (GtkWidget *table, int row, const setting *set)
 	{
 		if (text[i][0] != 0)
 		{
-			wid = gtk_radio_button_new_with_mnemonic (group, text[i]);
+			wid = gtk_radio_button_new_with_mnemonic (group, _(text[i]));
 			/*if (set->tooltip)
 				add_tip (wid, _(set->tooltip));*/
 			group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (wid));
@@ -867,22 +870,33 @@ setup_browsefile_cb (GtkWidget *button, GtkWidget *entry)
 	gtkutil_file_req (_("Select an Image File"), setup_filereq_cb, entry, NULL, 0);
 }
 
+static void
+setup_fontsel_destroy (GtkWidget *button, GtkFontSelectionDialog *dialog)
+{
+	font_dialog = NULL;
+}
+
 static void
 setup_fontsel_cb (GtkWidget *button, GtkFontSelectionDialog *dialog)
 {
 	GtkWidget *entry;
+	char *font_name;
 
 	entry = g_object_get_data (G_OBJECT (button), "e");
+	font_name = gtk_font_selection_dialog_get_font_name (dialog);
 
-	gtk_entry_set_text (GTK_ENTRY (entry),
-							  gtk_font_selection_dialog_get_font_name (dialog));
+	gtk_entry_set_text (GTK_ENTRY (entry), font_name);
+
+	g_free (font_name);
 	gtk_widget_destroy (GTK_WIDGET (dialog));
+	font_dialog = NULL;
 }
 
 static void
 setup_fontsel_cancel (GtkWidget *button, GtkFontSelectionDialog *dialog)
 {
 	gtk_widget_destroy (GTK_WIDGET (dialog));
+	font_dialog = NULL;
 }
 
 static void
@@ -898,6 +912,7 @@ setup_browsefont_cb (GtkWidget *button, GtkWidget *entry)
 	GtkFontSelectionDialog *dialog;
 
 	dialog = (GtkFontSelectionDialog *) gtk_font_selection_dialog_new (_("Select font"));
+	font_dialog = (GtkWidget *)dialog;	/* global var */
 
 	sel = (GtkFontSelection *) dialog->fontsel;
 
@@ -906,6 +921,8 @@ setup_browsefont_cb (GtkWidget *button, GtkWidget *entry)
 
 	g_object_set_data (G_OBJECT (dialog->ok_button), "e", entry);
 
+	g_signal_connect (G_OBJECT (dialog), "destroy",
+							G_CALLBACK (setup_fontsel_destroy), dialog);
 	g_signal_connect (G_OBJECT (dialog->ok_button), "clicked",
 							G_CALLBACK (setup_fontsel_cb), dialog);
 	g_signal_connect (G_OBJECT (dialog->cancel_button), "clicked",
@@ -1933,10 +1950,7 @@ setup_apply_real (int new_pix, int do_ulist, int do_layout)
 			setup_apply_to_sess (sess->gui);
 		}
 
-		if (prefs.logging)
-			log_open (sess);
-		else
-			log_close (sess);
+		log_open_or_close (sess);
 
 		if (do_ulist)
 			userlist_rehash (sess);
@@ -2091,6 +2105,12 @@ static void
 setup_close_cb (GtkWidget *win, GtkWidget **swin)
 {
 	*swin = NULL;
+
+	if (font_dialog)
+	{
+		gtk_widget_destroy (font_dialog);
+		font_dialog = NULL;
+	}
 }
 
 void
