@@ -428,7 +428,7 @@ userlist_dnd_motion (GtkTreeView *widget, GdkDragContext *context, gint x,
 		gtk_tree_selection_select_path (sel, path);
 	}
 
-	return TRUE;
+	return FALSE;
 }
 
 static gboolean
@@ -452,12 +452,16 @@ userlist_add_columns (GtkTreeView * treeview)
 
 	/* icon column */
 	renderer = gtk_cell_renderer_pixbuf_new ();
+	if (prefs.gui_tweaks & 32)
+		g_object_set (G_OBJECT (renderer), "ypad", 0, NULL);
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
 																-1, NULL, renderer,
 																"pixbuf", 0, NULL);
 
 	/* nick column */
 	renderer = gtk_cell_renderer_text_new ();
+	if (prefs.gui_tweaks & 32)
+		g_object_set (G_OBJECT (renderer), "ypad", 0, NULL);
 	gtk_cell_renderer_text_set_fixed_height_from_font (GTK_CELL_RENDERER_TEXT (renderer), 1);
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
 																-1, NULL, renderer,
@@ -467,6 +471,8 @@ userlist_add_columns (GtkTreeView * treeview)
 	{
 		/* hostname column */
 		renderer = gtk_cell_renderer_text_new ();
+		if (prefs.gui_tweaks & 32)
+			g_object_set (G_OBJECT (renderer), "ypad", 0, NULL);
 		gtk_cell_renderer_text_set_fixed_height_from_font (GTK_CELL_RENDERER_TEXT (renderer), 1);
 		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
 																	-1, NULL, renderer,
@@ -602,24 +608,25 @@ userlist_create (GtkWidget *box)
 										  GTK_SELECTION_MULTIPLE);
 
 	/* set up drops */
-/*	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL, dnd_dest_targets, 2,
-							 GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);*/
-	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL, dnd_dest_targets + 1, 1,
+	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL, dnd_dest_targets, 2,
 							 GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
 	gtk_drag_source_set (treeview, GDK_BUTTON1_MASK, dnd_src_target, 1, GDK_ACTION_MOVE);
 
-/*	g_signal_connect (G_OBJECT (treeview), "drag_motion",
+	/* file DND (for DCC) */
+	g_signal_connect (G_OBJECT (treeview), "drag_motion",
 							G_CALLBACK (userlist_dnd_motion), treeview);
 	g_signal_connect (G_OBJECT (treeview), "drag_leave",
 							G_CALLBACK (userlist_dnd_leave), 0);
 	g_signal_connect (G_OBJECT (treeview), "drag_data_received",
-							G_CALLBACK (userlist_dnd_drop), treeview);*/
+							G_CALLBACK (userlist_dnd_drop), treeview);
+
 	g_signal_connect (G_OBJECT (treeview), "button_press_event",
 							G_CALLBACK (userlist_click_cb), 0);
 	g_signal_connect (G_OBJECT (treeview), "key_press_event",
 							G_CALLBACK (userlist_key_cb), 0);
 
-#ifndef WIN32
+	/* tree/chanview DND */
+#ifndef WIN32	/* leaks GDI pool memory, don't enable */
 	g_signal_connect (G_OBJECT (treeview), "drag_begin",
 							G_CALLBACK (mg_drag_begin_cb), NULL);
 	g_signal_connect (G_OBJECT (treeview), "drag_drop",
