@@ -1,4 +1,5 @@
-typedef int (*socket_callback) (void *user_data, int sok);
+
+typedef int (*socket_callback) (void *source, int condition, void *user_data);
 typedef int (*timer_callback) (void *user_data);
 
 struct socketeventRec
